#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}


DOCUMENTATION = r'''
---
module: intersight_syslog_policy
short_description: Syslog Policy configuration for Cisco Intersight
description:
  - Manages Syslog Policy configuration on Cisco Intersight.
  - This policy configures local logging severity and up to two remote syslog server destinations for managed devices.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs/syslog/Policy/get/).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
    type: str
    default: default
  name:
    description:
      - The name assigned to the Syslog Policy.
    type: str
    required: true
  description:
    description:
      - The user-defined description for the Syslog Policy.
    type: str
    aliases: [descr]
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  local_logging_minimum_severity:
    description:
      - Lowest level of messages to be included in the local log.
    type: str
    choices: ['warning', 'emergency', 'alert', 'critical', 'error', 'notice', 'informational', 'debug']
    default: 'warning'
  first_remote_logging_enabled:
    description:
      - If C(true), enables the first remote syslog server destination.
    type: bool
    default: false
  first_remote_logging_hostname:
    description:
      - Hostname or IP Address of the first syslog server where log should be stored.
      - This parameter is required if C(first_remote_logging_enabled) is C(true).
    type: str
  first_remote_logging_port:
    description:
      - Port number used for logging on first syslog server.
    type: int
    default: 514
  first_remote_logging_protocol:
    description:
      - Transport layer protocol for transmission of log messages to first syslog server.
    type: str
    choices: ['udp', 'tcp']
    default: 'udp'
  first_remote_logging_minimum_severity:
    description:
      - Lowest level of messages to be included in the first remote log.
    type: str
    choices: ['warning', 'emergency', 'alert', 'critical', 'error', 'notice', 'informational', 'debug']
    default: 'warning'
  second_remote_logging_enabled:
    description:
      - If C(true), enables the second remote syslog server destination.
    type: bool
    default: false
  second_remote_logging_hostname:
    description:
      - Hostname or IP Address of the second syslog server where log should be stored.
      - This parameter is required if C(second_remote_logging_enabled) is C(true).
    type: str
  second_remote_logging_port:
    description:
      - Port number used for logging on second syslog server.
    type: int
    default: 514
  second_remote_logging_protocol:
    description:
      - Transport layer protocol for transmission of log messages to second syslog server.
    type: str
    choices: ['udp', 'tcp']
    default: 'udp'
  second_remote_logging_minimum_severity:
    description:
      - Lowest level of messages to be included in the second remote log.
    type: str
    choices: ['warning', 'emergency', 'alert', 'critical', 'error', 'notice', 'informational', 'debug']
    default: 'warning'
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Create a Syslog Policy with one remote server enabled
  cisco.intersight.intersight_syslog_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "Syslog-Policy-PROD"
    description: "Syslog policy for production servers"
    tags:
      - Key: "Env"
        Value: "Production"
    local_logging_minimum_severity: "notice"
    first_remote_logging_enabled: true
    first_remote_logging_hostname: "10.10.10.50"
    first_remote_logging_port: 514
    first_remote_logging_protocol: "udp"
    first_remote_logging_minimum_severity: "informational"

- name: Create a Syslog Policy with only local logging configured
  cisco.intersight.intersight_syslog_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "Syslog-Policy-Local-Only"
    description: "Send only critical local logs"
    local_logging_minimum_severity: "critical"

- name: Delete a Syslog Policy
  cisco.intersight.intersight_syslog_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "Syslog-Policy-PROD"
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "Syslog-Policy-Local-Only",
        "ObjectType": "syslog.Policy",
        "Tags": [],
        "LocalClients": [{
        "ClassId": "syslog.LocalFileLoggingClient",
        "ObjectType": "syslog.LocalFileLoggingClient",
        "MinSeverity": "critical"
        }],
    }
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def validate_input(module: AnsibleModule):
    first_remote_logging_port = module.params['first_remote_logging_port']
    second_remote_logging_port = module.params['second_remote_logging_port']

    if 0 > first_remote_logging_port or first_remote_logging_port > 65535:
        module.fail_json(msg=f"first_remote_logging_port has to be between 0 and 65535, current value is: {first_remote_logging_port}")

    if 0 > second_remote_logging_port or second_remote_logging_port > 65535:
        module.fail_json(msg=f"second_remote_logging_port has to be between 0 and 65535, current value is: {second_remote_logging_port}")


def customize_hostname_default(intersight: IntersightModule):
    # Create a custom default
    if not intersight.module.params['first_remote_logging_hostname']:
        intersight.module.params['first_remote_logging_hostname'] = "0.0.0.0"

    if not intersight.module.params['second_remote_logging_hostname']:
        intersight.module.params['second_remote_logging_hostname'] = "0.0.0.0"


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        local_logging_minimum_severity=dict(
            type='str',
            choices=['warning', 'emergency', 'alert', 'critical', 'error', 'notice', 'informational', 'debug'],
            default='warning'
        ),
        first_remote_logging_enabled=dict(type='bool', default=False),
        first_remote_logging_hostname=dict(type='str'),
        first_remote_logging_port=dict(type='int', default=514),
        first_remote_logging_protocol=dict(
            type='str',
            choices=['udp', 'tcp'],
            default='udp'
        ),
        first_remote_logging_minimum_severity=dict(
            type='str',
            choices=['warning', 'emergency', 'alert', 'critical', 'error', 'notice', 'informational', 'debug'],
            default='warning'
        ),
        second_remote_logging_enabled=dict(type='bool', default=False),
        second_remote_logging_hostname=dict(type='str'),
        second_remote_logging_port=dict(type='int', default=514),
        second_remote_logging_protocol=dict(
            type='str',
            choices=['udp', 'tcp'],
            default='udp'
        ),
        second_remote_logging_minimum_severity=dict(
            type='str',
            choices=['warning', 'emergency', 'alert', 'critical', 'error', 'notice', 'informational', 'debug'],
            default='warning'
        )

    )
    required_if = [
        ('first_remote_logging_enabled', True, ['first_remote_logging_hostname']),
        ('second_remote_logging_enabled', True, ['second_remote_logging_hostname'])
    ]

    module = AnsibleModule(
        argument_spec,
        required_if=required_if,
        supports_check_mode=True,
    )

    validate_input(module)
    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    customize_hostname_default(intersight)

    # Resource path used to configure policy
    resource_path = '/syslog/Policies'
    # Define API body used in compares or create
    intersight.api_body = {
        'Organization': {
            'Name': intersight.module.params['organization'],
        },
        'Name': intersight.module.params['name'],
        'LocalClients': [
            {
                'ObjectType': 'syslog.LocalFileLoggingClient',
                "MinSeverity": intersight.module.params['local_logging_minimum_severity']
            }
        ],
        'RemoteClients': [
            {
                'Type': 'syslog.RemoteLoggingClient',
                'Enabled': intersight.module.params['first_remote_logging_enabled'],
                'ObjectType': 'syslog.RemoteLoggingClient',
                'Hostname': intersight.module.params['first_remote_logging_hostname'],
                'Port': intersight.module.params['first_remote_logging_port'],
                'Protocol': intersight.module.params['first_remote_logging_protocol'],
                'MinSeverity': intersight.module.params['first_remote_logging_minimum_severity']
            },
            {
                'Type': 'syslog.RemoteLoggingClient',
                'Enabled': intersight.module.params['second_remote_logging_enabled'],
                'ObjectType': 'syslog.RemoteLoggingClient',
                'Hostname': intersight.module.params['second_remote_logging_hostname'],
                'Port': intersight.module.params['second_remote_logging_port'],
                'Protocol': intersight.module.params['second_remote_logging_protocol'],
                'MinSeverity': intersight.module.params['second_remote_logging_minimum_severity']
            }
        ],
    }

    intersight.set_tags_and_description()

    intersight.configure_policy_or_profile(resource_path=resource_path)

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright (c) 2022 Mandar Kulkarni, < @mandar242 >
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_account_info

version_added: "1.14.0"

short_description: Get Azure Account facts (output of az account show)

description:
    - Get facts for current logged in user.
    - Output equivalent of `az account show` command.

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Mandar Kulkarni (@mandar242)
'''

EXAMPLES = '''
- name: Get facts for current logged in user
  azure.azcollection.azure_rm_account_info:
'''

RETURN = '''
account_info:
    description:
        - Facts for current logged in user, equivalent to `az account show`.
    returned: always
    type: dict
    contains:
        environmentName:
            description: For cloud environments other than the US public cloud, the environment name.
            returned: always
            type: str
            sample: AzureCloud
        homeTenantId:
            description: Subscription tenant id.
            returned: always
            type: str
            sample: "00000000-0000-0000-0000-000000000000"
        id:
            description: Subscription id.
            returned: always
            type: str
            sample: "00000000-0000-0000-0000-000000000000"
        managedByTenants:
            description: An array containing the tenants managing the subscription.
            returned: always
            type: list
            elements: dict
            contains:
                tenantId:
                    description: Subscription tenant id
                    returned: always
                    type: str
                    sample: "00000000-0000-0000-0000-000000000000"
        name:
            description: The subscription display name.
            returned: always
            type: str
            sample: "Pay-As-You-Go"
        state:
            description:
                - The subscription state.
                - Possible values include "Enabled", "Warned", "PastDue", "Disabled", "Deleted".
            returned: always
            type: str
            sample: "Enabled"
        tenant_id:
            description: Subscription tenant id
            returned: always
            type: str
            sample: "00000000-0000-0000-0000-000000000000"
        user:
            description: An dict containing the current user name and type.
            returned: always
            type: dict
            elements: str
            contains:
                name:
                    description: The principal name of the active directory user.
                    returned: always
                    type: str
                    sample: "sample-user@sample-tenant.onmicrosoft.com"
                type:
                    description: Active Directory user type.
                    returned: always
                    type: str
                    sample: "User"
'''


try:
    import asyncio
    from msgraph.generated.education.me.user.user_request_builder import UserRequestBuilder
except ImportError:
    # This is handled in azure_rm_common
    pass

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase


class AzureRMAccountInfo(AzureRMModuleBase):

    def __init__(self):

        self.module_arg_spec = dict(
        )

        self.results = dict(
            changed=False,
            account_info=[]
        )

        # Different return info is gathered using 2 different clients
        # 1. All except "user" section of the return value uses azure.mgmt.subsctiption.operations.subscriptionoperations
        # 2. "user" section of the return value uses different client (GraphServiceClient)

        super(AzureRMAccountInfo, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                 supports_check_mode=True,
                                                 supports_tags=False,
                                                 is_ad_resource=True)

    def exec_module(self, **kwargs):
        self.results['account_info'] = self.list_items()
        return self.results

    def list_items(self):

        results = {}

        # Get
        # "homeTenantId": "xxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxx",
        # "id": "xxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxx",
        # "isDefault": true,                                    <- WIP on getting this param
        # "managedByTenants": [
        #     {
        #     "tenantId": "64xxxxxx-xxxx-49fc-xxxx-ebxxxxxxxxxx"
        #     },
        #     {
        #     "tenantId": "2axxxxxx-xxxx-xxxx-a339-ebxxxxxxxxxx"
        #     },
        #     {
        #     "tenantId": "xxxxxxxx-xxxx-4e68-xxxx-ebxxxxxxxxxx"
        #     }
        # ],
        # "name": "Pay-As-You-Go",
        # "state": "Enabled",
        # "tenantId": "xxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxx",

        # Makes use of azure.mgmt.subsctiption.operations.subscriptionoperations
        # https://docs.microsoft.com/en-us/python/api/azure-mgmt-subscription/azure.mgmt.subscription.operations.subscriptionsoperations?view=azure-python#methods

        try:
            subscription_list_response = list(self.subscription_client.subscriptions.list())
        except Exception as exc:
            self.fail("Failed to list all subscriptions - {0}".format(str(exc)))

        results['id'] = subscription_list_response[0].subscription_id
        results['tenantId'] = subscription_list_response[0].tenant_id
        results['homeTenantId'] = subscription_list_response[0].tenant_id
        results['name'] = subscription_list_response[0].display_name
        results['state'] = subscription_list_response[0].state
        results['managedByTenants'] = self.get_managed_by_tenants_list(subscription_list_response[0].managed_by_tenants)
        results['environmentName'] = self.azure_auth._cloud_environment.name
        results['user'] = self.get_aduser_info()

        return results

    def get_managed_by_tenants_list(self, object_list):

        return [dict(tenantId=item.tenant_id) for item in object_list]

    def get_aduser_info(self):

        # Create GraphServiceClient for getting
        # "user": {
        #     "name": "mandar123456@abcdefg.onmicrosoft.com",
        #     "type": "Member"
        # }

        # Makes use of azure MSGraph
        # https://learn.microsoft.com/en-us/graph/api/user-get?view=graph-rest-1.0&tabs=http

        user = {}

        user_info = asyncio.get_event_loop().run_until_complete(self.getAccount())
        user['name'] = user_info.user_principal_name
        user['type'] = user_info.user_type
        return user

    async def getAccount(self):
        return await self.get_msgraph_client().me.get(
            request_configuration=UserRequestBuilder.UserRequestBuilderGetRequestConfiguration(
                query_parameters=UserRequestBuilder.UserRequestBuilderGetQueryParameters(
                    select=["userType", "userPrincipalName", "postalCode", "identities"], ),
            )
        )


def main():
    AzureRMAccountInfo()


if __name__ == '__main__':
    main()



#
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The eos ospfv2 fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""
import re

from copy import deepcopy

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import utils

from ansible_collections.arista.eos.plugins.module_utils.network.eos.argspec.ospfv2.ospfv2 import (
    Ospfv2Args,
)


class Ospfv2Facts(object):
    """The eos ospfv2 fact class"""

    def __init__(self, module, subspec="config", options="options"):
        self._module = module
        self.argument_spec = Ospfv2Args.argument_spec
        spec = deepcopy(self.argument_spec)
        if subspec:
            if options:
                facts_argument_spec = spec[subspec][options]
            else:
                facts_argument_spec = spec[subspec]
        else:
            facts_argument_spec = spec

        self.generated_spec = utils.generate_dict(facts_argument_spec)

    def get_device_data(self, connection):
        return connection.get("show running-config | section ospf")

    def populate_facts(self, connection, ansible_facts, data=None):
        """Populate the facts for ospfv2
        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf
        :rtype: dictionary
        :returns: facts
        """

        if not data:
            data = self.get_device_data(connection)

        # split the config into instances of the resource
        resource_delim = "router ospf"
        find_pattern = r"(?:^|\n)%s.*?(?=(?:^|\n)%s|$)" % (
            resource_delim,
            resource_delim,
        )
        resources = [p.strip() for p in re.findall(find_pattern, data, re.DOTALL)]
        objs_list = []
        objs = {}
        for resource in resources:
            if resource and "router ospfv3" not in resource:
                obj = self.render_config(self.generated_spec, resource)
                if obj:
                    objs_list.append(obj)
        objs = {"processes": objs_list}
        ansible_facts["ansible_network_resources"].pop("ospfv2", None)

        facts = {}
        if objs:
            facts["ospfv2"] = {}
            params = utils.validate_config(
                self.argument_spec,
                {"config": objs},
            )
            facts["ospfv2"].update(utils.remove_empties(params["config"]))

        ansible_facts["ansible_network_resources"].update(facts)
        return ansible_facts

    def render_config(self, spec, conf):
        """
        Render config as dictionary structure and delete keys
          from spec for null values

        :param spec: The facts tree, generated from the argspec
        :param conf: The configuration
        :rtype: dictionary
        :returns: The generated config
        """
        instance_list = []
        ospf_params_dict = {}
        areas_list = []
        distance_dict = {}
        network_list = []
        redistribute_list = []
        timers_list = []
        areas_list = []
        for dev_config in conf.split("\n"):
            if not dev_config:
                continue
            network_dict = {}
            redistribute_dict = {}
            dev_config = dev_config.strip()
            dev_config = re.sub(r"-", "_", dev_config).strip()
            matches = re.findall(r"router (ospf) (.*)", dev_config)
            if matches:
                if ospf_params_dict:
                    instance_list.append(ospf_params_dict)
                    ospf_params_dict = {}
                instance = matches[0][1].split()
                ospf_params_dict.update({"process_id": str(instance[0])})
                if "vrf" in dev_config:
                    vrf_name = instance[-1]
                else:
                    vrf_name = None
                ospf_params_dict.update({"vrf": vrf_name})
            if "traffic_engineering" in dev_config:
                ospf_params_dict.update({"traffic_engineering": True})
            config_params = dev_config.split()
            if config_params[0] == "adjacency":
                threshold = config_params[-1]
                adjacency_dict = {"exchange_start": {"threshold": threshold}}
                ospf_params_dict.update({"adjacency": adjacency_dict})
            elif "auto_cost" in dev_config:
                bw = config_params[-1]
                ospf_params_dict.update(
                    {"auto_cost": {"reference_bandwidth": bw}},
                )
            elif "bfd" in dev_config:
                ospf_params_dict.update({"bfd": {"all_interfaces": True}})
            elif config_params[0] == "default_information":
                def_dict = {"originate": True}
                for i, val in enumerate(config_params[2::]):
                    if val == "always":
                        def_dict.update({"always": True})
                    elif val in ["route_map", "metric", "metric_type"]:
                        def_dict.update({val: config_params[i + 3]})
                ospf_params_dict.update({"default_information": def_dict})
            elif "default_metric" in dev_config:
                ospf_params_dict.update({"default_metric": config_params[-1]})
            elif "distance" in dev_config:
                distance_dict.update({config_params[-2]: config_params[-1]})
                ospf_params_dict.update({"distance": distance_dict})
            elif "distribute_list" in dev_config:
                ospf_params_dict.update(
                    {"distribute_list": {config_params[1]: config_params[2]}},
                )
            elif "dn_bit_ignore" in dev_config:
                ospf_params_dict.update({"dn_bit_ignore": True})
            elif "fips_restrictions" in dev_config:
                ospf_params_dict.update({"fips_restrictions": True})
            elif "graceful_restart" in dev_config:
                if "grace_period" in dev_config:
                    ospf_params_dict.update(
                        {
                            "graceful_restart": {
                                "grace_period": config_params[-1],
                            },
                        },
                    )
                else:
                    ospf_params_dict.update(
                        {"graceful_restart": {"set": True}},
                    )
            elif "graceful_restart_helper" in dev_config:
                ospf_params_dict.update({"graceful_restart_helper": True})
            elif "log_adjacency_changes" in dev_config:
                detail = True if "detail" in dev_config else False
                ospf_params_dict.update(
                    {"log_adjacency_changes": {"detail": detail}},
                )
            elif "max_lsa" in dev_config:
                max_lsa_dict = {}
                config_params.pop(0)
                max_lsa_dict.update({"count": config_params.pop(0)})
                if config_params:
                    if config_params[0].isdigit():
                        max_lsa_dict.update(
                            {"threshold": config_params.pop(0)},
                        )
                    for i, el in enumerate(config_params):
                        if el == "warning_only":
                            max_lsa_dict.update({"warning": True})
                        if el in ["ignore_count", "ignore_time", "reset_time"]:
                            max_lsa_dict.update({el: config_params[i + 1]})
                ospf_params_dict.update({"max_lsa": max_lsa_dict})
            elif "maximum_paths" in dev_config:
                ospf_params_dict.update({"maximum_paths": config_params[1]})
            elif "mpls ldp sync default" in dev_config:
                ospf_params_dict.update({"mpls_ldp": True})
            elif config_params[0] == "network":
                config_params.pop(0)
                prefix = re.search(r"\/", config_params[0])
                if prefix:
                    network_dict.update({"prefix": config_params.pop(0)})
                else:
                    network_dict.update(
                        {"network_address": config_params.pop(0)},
                    )
                    network_dict.update({"mask": config_params.pop(0)})
                network_dict.update({"area": config_params[-1]})
                network_list.append(network_dict)
                ospf_params_dict.update({"networks": network_list})
            elif "passive_interface" in dev_config:
                if config_params[1] == "default":
                    ospf_params_dict.update(
                        {"passive_interface": {"default": True}},
                    )
                else:
                    ospf_params_dict.update(
                        {
                            "passive_interface": {
                                "interface_list": config_params[1],
                            },
                        },
                    )
            elif "point_to_point" in dev_config:
                ospf_params_dict.update({"point_to_point": True})
            elif "redistribute" in dev_config:
                redistribute_dict.update({"routes": config_params[1]})
                if config_params[1] == "isis":
                    if "level" in config_params[2]:
                        k = re.sub(r"_", "-", config_params[2])
                        redistribute_dict.update({"isis_level": k})
                if "route_map" in dev_config:
                    redistribute_dict.update({"route_map": config_params[-1]})
                redistribute_list.append(redistribute_dict)
                ospf_params_dict.update({"redistribute": redistribute_list})
            elif "router_id" in dev_config:
                ospf_params_dict.update({"router_id": config_params[-1]})
            elif "retransmission_threshold" in dev_config:
                ospf_params_dict.update(
                    {"retransmission_threshold": config_params[-1]},
                )
            elif config_params[0] == "compatible":
                ospf_params_dict.update({"rfc1583compatibility": True})
            elif "shutdown" in dev_config:
                ospf_params_dict.update({"shutdown": True})
            elif "summary_address" in dev_config:
                summary_address_dict = {}
                config_params.pop(0)
                prefix = re.search(r"\/", config_params[0])
                if prefix:
                    summary_address_dict.update(
                        {"prefix": config_params.pop(0)},
                    )
                else:
                    summary_address_dict.update(
                        {"address": config_params.pop(0)},
                    )
                    summary_address_dict.update({"mask": config_params.pop(0)})
                if "not_advertise" in dev_config:
                    summary_address_dict.update({"not_advertise": True})
                    config_params.pop(0)
                else:
                    if config_params:
                        summary_address_dict.update(
                            {config_params[0]: config_params[1]},
                        )
                ospf_params_dict.update(
                    {"summary_address": summary_address_dict},
                )
            elif "timers" in dev_config:
                timers_dict = {}
                if config_params[1] == "lsa":
                    if config_params[2] in ["rx", "arrival"]:
                        timers_dict.update(
                            {
                                "lsa": {
                                    "rx": {"min_interval": config_params[-1]},
                                },
                            },
                        )
                    else:
                        timers_dict.update(
                            {
                                "lsa": {
                                    "tx": {
                                        "delay": {
                                            "initial": config_params[-3],
                                            "min": config_params[-2],
                                            "max": config_params[-1],
                                        },
                                    },
                                },
                            },
                        )
                elif config_params[1] == "out_delay":
                    timers_dict.update({"out_delay": config_params[-1]})
                elif config_params[1] == "pacing":
                    timers_dict.update({"pacing": config_params[-1]})
                elif config_params[1] == "spf":
                    if config_params[2] == "delay":
                        timers_dict.update(
                            {
                                "spf": {
                                    "tx": {
                                        "delay": {
                                            "initial": config_params[-3],
                                            "min": config_params[-2],
                                            "max": config_params[-1],
                                        },
                                    },
                                },
                            },
                        )
                    else:
                        timers_dict.update(
                            {"spf": {"seconds": config_params[-1]}},
                        )
                elif config_params[1] == "throttle":
                    timers_dict.update(
                        {
                            "throttle": {
                                "attr": config_params[2],
                                "initial": config_params[-3],
                                "min": config_params[-2],
                                "max": config_params[-1],
                            },
                        },
                    )
                timers_list.append(timers_dict)
                ospf_params_dict.update({"timers": timers_list})
            elif config_params[0] == "area":
                areas_dict = {}
                areas_dict.update({"area_id": config_params[1]})
                if config_params[2] == "default_cost":
                    areas_dict.update({"default_cost": config_params[-1]})
                elif config_params[2] == "filter":
                    prefix = re.search(r"\/", config_params[3])
                    if prefix:
                        areas_dict.update(
                            {"filter": {"address": config_params[3]}},
                        )
                    elif config_params[3] == "prefix_list":
                        areas_dict.update(
                            {"filter": {"prefix_list": config_params[-1]}},
                        )
                    else:
                        areas_dict.update(
                            {"filter": {"subnet_address": config_params[3]}},
                        )
                        areas_dict.update(
                            {"filter": {"subnet_mask": config_params[4]}},
                        )
                elif config_params[2] == "not_so_stubby":
                    if len(config_params) == 3:
                        areas_dict.update({"not_so_stubby": {"set": True}})
                        continue
                    if config_params[3] == "lsa":
                        areas_dict.update({"not_so_stubby": {"lsa": True}})
                    elif config_params[3] == "default_information_originate":
                        default_dict = {}
                        for i, val in enumerate(config_params):
                            if val == "nssa_only":
                                default_dict.update({"nssa_only": True})
                            if val == "metric_type":
                                default_dict.update(
                                    {"metric_type": config_params[i + 1]},
                                )
                            if val == "metric":
                                default_dict.update(
                                    {"metric": config_params[i + 1]},
                                )
                        areas_dict.update(
                            {
                                "not_so_stubby": {
                                    "default_information_originate": default_dict,
                                },
                            },
                        )
                    elif config_params[3] == "no_summary":
                        areas_dict.update(
                            {"not_so_stubby": {"no_summary": True}},
                        )
                    elif config_params[3] == "nssa_only":
                        areas_dict.update(
                            {"not_so_stubby": {"nssa_only": True}},
                        )
                elif config_params[2] == "nssa":
                    if len(config_params) == 3:
                        areas_dict.update({"nssa": {"set": True}})
                        continue
                    if config_params[3] == "default_information_originate":
                        default_dict = {}
                        for i, val in enumerate(config_params):
                            if val == "nssa_only":
                                default_dict.update({"nssa_only": True})
                            if val == "metric_type":
                                default_dict.update(
                                    {"metric_type": config_params[i + 1]},
                                )
                            if val == "metric":
                                default_dict.update(
                                    {"metric": config_params[i + 1]},
                                )
                        areas_dict.update(
                            {
                                "nssa": {
                                    "default_information_originate": default_dict,
                                },
                            },
                        )
                    elif config_params[3] == "no_summary":
                        areas_dict.update({"nssa": {"no_summary": True}})
                    elif config_params[3] == "nssa_only":
                        areas_dict.update({"nssa": {"nssa_only": True}})
                elif config_params[2] == "range":
                    prefix = re.search(r"\/", config_params[3])
                    range_dict = {}
                    if prefix:
                        range_dict.update({"address": config_params[3]})
                    else:
                        range_dict.update({"subnet_address": config_params[3]})
                        range_dict.update({"subnet_mask": config_params[4]})
                    if "advertise" in dev_config:
                        range_dict.update({"advertise": True})
                    if "not_advertise" in dev_config:
                        range_dict.update({"advertise": False})
                    if "cost" in dev_config:
                        range_dict.update({"cost": config_params[-1]})
                    areas_dict.update({"range": range_dict})
                elif config_params[2] == "stub":
                    if "no_summary" in dev_config:
                        areas_dict.update({"stub": {"no_summary": True}})
                    else:
                        areas_dict.update({"stub": {"set": True}})
                areas_list.append(areas_dict)
                ospf_params_dict.update({"areas": areas_list})
            elif config_params[0] == "max_metric":
                config_params.pop(0)
                router_lsa_dict = {}
                config_params.pop(0)
                if not config_params:
                    ospf_params_dict.update(
                        {"max_metric": {"router_lsa": {"set": True}}},
                    )
                else:
                    for i, val in enumerate(config_params):
                        if val == "include_stub":
                            router_lsa_dict.update({"include_stub": True})
                        elif val == "on_startup":
                            if config_params[i + 1] == "wait_for_bgp":
                                router_lsa_dict.update(
                                    {"on_startup": {"wait_for_bgp": True}},
                                )
                            else:
                                router_lsa_dict.update(
                                    {
                                        "on_startup": {
                                            "time": config_params[i + 1],
                                        },
                                    },
                                )
                        elif val == "external_lsa":
                            if i < len(config_params) and config_params[i + 1].isdigit():
                                router_lsa_dict.update(
                                    {
                                        "external_lsa": {
                                            "max_metric_value": config_params[i + 1],
                                        },
                                    },
                                )
                            else:
                                router_lsa_dict.update(
                                    {"external_lsa": {"set": True}},
                                )
                        elif val == "summary_lsa":
                            if i < len(config_params) - 1 and config_params[i + 1].isdigit():
                                router_lsa_dict.update(
                                    {
                                        "summary_lsa": {
                                            "max_metric_value": config_params[i + 1],
                                        },
                                    },
                                )
                            else:
                                router_lsa_dict.update(
                                    {"summary_lsa": {"set": True}},
                                )
                    ospf_params_dict.update(
                        {"max_metric": {"router_lsa": router_lsa_dict}},
                    )
        # instance_list.append(ospf_params_dict)
        # config.update({"ospf_version": "v2", "ospf_processes": instance_list})
        return utils.remove_empties(ospf_params_dict)



# Copyright (c) 2016, Yanis Guenane <yanis+ansible@guenane.org>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this doc fragment is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations


class ModuleDocFragment:
    # Standard files documentation fragment
    DOCUMENTATION = r"""
description:
  - One can generate L(RSA,https://en.wikipedia.org/wiki/RSA_%28cryptosystem%29), L(DSA,https://en.wikipedia.org/wiki/Digital_Signature_Algorithm),
    L(ECC,https://en.wikipedia.org/wiki/Elliptic-curve_cryptography) or L(EdDSA,https://en.wikipedia.org/wiki/EdDSA) private
    keys.
  - Keys are generated in PEM format.
attributes:
  diff_mode:
    support: full
  idempotent:
    support: partial
    details:
      - The option O(regenerate=always) generally disables idempotency.
requirements:
  - cryptography >= 3.3
options:
  size:
    description:
      - Size (in bits) of the TLS/SSL key to generate.
    type: int
    default: 4096
  type:
    description:
      - The algorithm used to generate the TLS/SSL private key.
    type: str
    default: RSA
    choices: [DSA, ECC, Ed25519, Ed448, RSA, X25519, X448]
  curve:
    description:
      - Note that not all curves are supported by all versions of C(cryptography).
      - For maximal interoperability, V(secp384r1) or V(secp256r1) should be used.
      - We use the curve names as defined in the L(IANA registry for TLS,https://www.iana.org/assignments/tls-parameters/tls-parameters.xhtml#tls-parameters-8).
      - Please note that all curves except V(secp224r1), V(secp256k1), V(secp256r1), V(secp384r1), and V(secp521r1) are discouraged
        for new private keys.
    type: str
    choices:
      - secp224r1
      - secp256k1
      - secp256r1
      - secp384r1
      - secp521r1
      - secp192r1
      - brainpoolP256r1
      - brainpoolP384r1
      - brainpoolP512r1
      - sect163k1
      - sect163r2
      - sect233k1
      - sect233r1
      - sect283k1
      - sect283r1
      - sect409k1
      - sect409r1
      - sect571k1
      - sect571r1
  passphrase:
    description:
      - The passphrase for the private key.
    type: str
  cipher:
    description:
      - The cipher to encrypt the private key. This is only used when O(passphrase) is provided.
      - Must be V(auto).
    type: str
    default: auto
  select_crypto_backend:
    description:
      - Determines which crypto backend to use.
      - The default choice is V(auto), which tries to use C(cryptography) if available.
      - If set to V(cryptography), will try to use the L(cryptography,https://cryptography.io/) library.
      - Note that with community.crypto 3.0.0, all values behave the same.
        This option will be deprecated in a later version.
        We recommend to not set it explicitly.
    type: str
    default: auto
    choices: [auto, cryptography]
  format:
    description:
      - Determines which format the private key is written in. By default, PKCS1 (traditional OpenSSL format) is used for
        all keys which support it. Please note that not every key can be exported in any format.
      - The value V(auto) selects a format based on the key format. The value V(auto_ignore) does the same, but for existing
        private key files, it will not force a regenerate when its format is not the automatically selected one for generation.
      - Note that if the format for an existing private key mismatches, the key is B(regenerated) by default. To change this
        behavior, use the O(format_mismatch) option.
    type: str
    default: auto_ignore
    choices: [pkcs1, pkcs8, raw, auto, auto_ignore]
  format_mismatch:
    description:
      - Determines behavior of the module if the format of a private key does not match the expected format, but all other
        parameters are as expected.
      - If set to V(regenerate) (default), generates a new private key.
      - If set to V(convert), the key will be converted to the new format instead.
    type: str
    default: regenerate
    choices: [regenerate, convert]
  regenerate:
    description:
      - Allows to configure in which situations the module is allowed to regenerate private keys. The module will always generate
        a new key if the destination file does not exist.
      - By default, the key will be regenerated when it does not match the module's options, except when the key cannot be
        read or the passphrase does not match. Please note that this B(changed) for Ansible 2.10. For Ansible 2.9, the behavior
        was as if V(full_idempotence) is specified.
      - If set to V(never), the module will fail if the key cannot be read or the passphrase is not matching, and will never
        regenerate an existing key.
      - If set to V(fail), the module will fail if the key does not correspond to the module's options.
      - If set to V(partial_idempotence), the key will be regenerated if it does not conform to the module's options. The
        key is B(not) regenerated if it cannot be read (broken file), the key is protected by an unknown passphrase, or when
        they key is not protected by a passphrase, but a passphrase is specified.
      - If set to V(full_idempotence), the key will be regenerated if it does not conform to the module's options. This is
        also the case if the key cannot be read (broken file), the key is protected by an unknown passphrase, or when they
        key is not protected by a passphrase, but a passphrase is specified. Make sure you have a B(backup) when using this
        option!
      - If set to V(always), the module will always regenerate the key. This is equivalent to setting O(force) to V(true).
      - Note that if O(format_mismatch) is set to V(convert) and everything matches except the format, the key will always
        be converted, except if O(regenerate) is set to V(always).
    type: str
    choices:
      - never
      - fail
      - partial_idempotence
      - full_idempotence
      - always
    default: full_idempotence
seealso:
  - module: community.crypto.x509_certificate
  - module: community.crypto.x509_certificate_pipe
  - module: community.crypto.openssl_csr
  - module: community.crypto.openssl_csr_pipe
  - module: community.crypto.openssl_dhparam
  - module: community.crypto.openssl_pkcs12
  - module: community.crypto.openssl_publickey
"""



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import time
import os
import json
import base64
import tarfile
from glob import glob
import traceback

HAS_ORAS = False
HAS_ORAS_EXC = None

try:
    import oras.client
    HAS_ORAS = True
except ImportError:
    oras = None
    HAS_ORAS_EXC = traceback.format_exc()

try:
    from azure.core.exceptions import ResourceNotFoundError, HttpResponseError
    from azure.mgmt.core.tools import resource_id, parse_resource_id
except ImportError:
    pass

from ansible_collections.azure.azcollection.plugins.plugin_utils import file_utils
from ansible_collections.azure.azcollection.plugins.plugin_utils import constants as consts

from ansible.errors import AnsibleError


# Get the Access Details to connect to Arc Connectivity platform from the HybridConnectivity RP
def get_relay_information(rest_client, subscription_id, resource_group, hostname, resource_type,
                          certificate_validity_in_seconds, port):
    if not certificate_validity_in_seconds or \
       certificate_validity_in_seconds > consts.RELAY_INFO_MAXIMUM_DURATION_IN_SECONDS:
        certificate_validity_in_seconds = consts.RELAY_INFO_MAXIMUM_DURATION_IN_SECONDS

    namespace = resource_type.split('/', 1)[0]
    arc_type = resource_type.split('/', 1)[1]
    resource_uri = resource_id(subscription=subscription_id, resource_group=resource_group,
                               namespace=namespace, type=arc_type, name=hostname)

    cred = None
    new_service_config = False
    try:
        cred = _list_credentials(rest_client, resource_uri, certificate_validity_in_seconds)
    except ResourceNotFoundError:
        _create_default_endpoint(rest_client, resource_uri)
    except HttpResponseError as e:
        if e.reason != "Precondition Failed":
            raise AnsibleError(f"Unable to retrieve relay information. Failed with error: {str(e)}")
    except Exception as e:
        raise AnsibleError(f"Unable to retrieve relay information. Failed with error: {str(e)}")

    if not cred:
        _create_service_configuration(rest_client, resource_uri, port)
        new_service_config = True
        try:
            cred = _list_credentials(rest_client, resource_uri, certificate_validity_in_seconds)
        except Exception as e:
            raise AnsibleError(f"Unable to get relay information. Failed with error: {str(e)}")
        _handle_relay_connection_delay(rest_client, "Setting up service configuration")
    else:
        if not _check_service_configuration(rest_client, resource_uri, port):
            _create_service_configuration(rest_client, resource_uri, port)
            new_service_config = True
            try:
                cred = _list_credentials(rest_client, resource_uri, certificate_validity_in_seconds)
            except Exception as e:
                raise AnsibleError(f"Unable to get relay information. Failed with error: {str(e)}")
            _handle_relay_connection_delay(rest_client, "Setting up service configuration")
    return (cred, new_service_config)


def _check_service_configuration(rest_client, resource_uri, port):
    url = f"/{resource_uri}/providers/Microsoft.HybridConnectivity/endpoints/default/serviceConfigurations/SSH"

    serviceConfig = None
    # pylint: disable=broad-except
    try:
        serviceConfig = resource(rest_client, url, "GET")
    except Exception:
        # If for some reason the request for Service Configuration fails,
        # we will still attempt to get relay information and connect. If the service configuration
        # is not setup correctly, the connection will fail.
        # The more likely scenario is that the request failed with a "Authorization Error",
        # in case the user isn't an owner/contributor.
        return True
    serviceConfigPort = serviceConfig and serviceConfig.get('properties', {}).get('port')
    if port:
        return int(serviceConfigPort) == int(port)

    return True


def _create_default_endpoint(rest_client, resource_uri):
    url = f"/{resource_uri}/providers/Microsoft.HybridConnectivity/endpoints/default"
    body = {'properties': {'type': 'default'}}

    hostname = parse_resource_id(resource_uri)["name"]
    resource_group = parse_resource_id(resource_uri)["resource_group"]
    try:
        endpoint = resource(rest_client, url, "PUT", body=body)
    except HttpResponseError as e:
        if e.reason == "Forbidden":
            raise AnsibleError(f"Client is not authorized to create a default connectivity "
                               f"endpoint for \'{hostname}\' in Resource Group \'{resource_group}\'. "
                               f"This is a one-time operation that must be performed by "
                               f"an account with Owner or Contributor role to allow "
                               f"connections to the target resource.")
        raise AnsibleError(f"Failed to create default endpoint for the target Arc Server. "
                           f"\nError: {str(e)}")
    except Exception as e:
        raise AnsibleError(f"Failed to create default endpoint for the target Arc Server. "
                           f"\nError: {str(e)}")

    return endpoint


def _create_service_configuration(rest_client, resource_uri, port):
    if not port:
        port = '22'

    url = f"/{resource_uri}/providers/Microsoft.HybridConnectivity/endpoints/default/serviceConfigurations/SSH"
    body = {'properties': {'port': int(port), 'serviceName': 'SSH'}}

    hostname = parse_resource_id(resource_uri)["name"]
    resource_group = parse_resource_id(resource_uri)["resource_group"]

    try:
        serviceConfig = resource(rest_client, url, "PUT", body=body)
    except HttpResponseError as e:
        if e.reason == "Forbidden":
            raise AnsibleError(f"Client is not authorized to create or update the Service "
                               f"Configuration endpoint for \'{hostname}\' in the Resource "
                               f"Group \'{resource_group}\'. This is an operation that "
                               f"must be performed by an account with Owner or Contributor "
                               f"role to allow SSH connections to the specified port {port}.")
        raise AnsibleError(f"Failed to create service configuration to allow SSH "
                           f"connections to port {port} on the endpoint for {hostname} "
                           f"in the Resource Group {resource_group}\nError: {str(e)}")
    except Exception as e:
        raise AnsibleError(f"Failed to create service configuration to allow SSH connections "
                           f"to port {port} on the endpoint for {hostname} in the Resource "
                           f"Group {resource_group}\nError: {str(e)}")
    return serviceConfig


def _list_credentials(rest_client, resource_uri, certificate_validity_in_seconds):
    url = f"/{resource_uri}/providers/Microsoft.HybridConnectivity/endpoints/default/listCredentials"
    query_parameters = {'expiresin': int(certificate_validity_in_seconds)}
    body = {'serviceName': 'SSH'}

    response = resource(rest_client, url, "POST", body=body, custom_query=query_parameters)
    return response and response.get('relay')


def format_relay_info_string(relay_info):
    relay_info_string = json.dumps(
        {
            "relay": {
                "namespaceName": relay_info['namespaceName'],
                "namespaceNameSuffix": relay_info['namespaceNameSuffix'],
                "hybridConnectionName": relay_info['hybridConnectionName'],
                "accessKey": relay_info['accessKey'],
                "expiresOn": relay_info['expiresOn'],
                "serviceConfigurationToken": relay_info['serviceConfigurationToken']
            }
        })
    result_bytes = relay_info_string.encode("ascii")
    enc = base64.b64encode(result_bytes)
    base64_result_string = enc.decode("ascii")
    return base64_result_string


def _handle_relay_connection_delay(rest_client, message):
    # relay has retry delay after relay connection is lost
    # must sleep for at least as long as the delay
    # otherwise the ssh connection will fail
    for x in range(0, consts.SERVICE_CONNECTION_DELAY_IN_SECONDS + 1):
        time.sleep(1)


# Downloads client side proxy to connect to Arc Connectivity Platform
def install_client_side_proxy(arc_proxy_folder):

    client_operating_system = _get_client_operating_system()
    client_architecture = _get_client_architeture()
    install_dir = _get_proxy_install_dir(arc_proxy_folder)
    proxy_name = _get_proxy_filename(client_operating_system, client_architecture)
    install_location = os.path.join(install_dir, proxy_name)

    # Only download new proxy if it doesn't exist already
    if not os.path.isfile(install_location):
        if not os.path.isdir(install_dir):
            file_utils.create_directory(install_dir, f"Failed to create client proxy directory '{install_dir}'.")
        # if directory exists, delete any older versions of the proxy
        else:
            older_version_location = _get_older_version_proxy_path(
                install_dir,
                client_operating_system,
                client_architecture)
            older_version_files = glob(older_version_location)
            for f in older_version_files:
                file_utils.delete_file(f, f"failed to delete older version file {f}", warning=True)

        _download_proxy_from_MCR(install_dir, proxy_name, client_operating_system, client_architecture)
        _check_proxy_installation(install_dir, proxy_name)

    return install_location


def _download_proxy_from_MCR(dest_dir, proxy_name, operating_system, architecture):
    mar_target = f"{consts.CLIENT_PROXY_MCR_TARGET}/{operating_system.lower()}/{architecture}/ssh-proxy"

    client = oras.client.OrasClient()

    try:
        response = client.pull(target=f"{mar_target}:{consts.CLIENT_PROXY_VERSION}", outdir=dest_dir)
    except Exception as e:
        raise AnsibleError(
            f"Failed to download Arc Connectivity proxy with error {str(e)}. Please try again.")

    proxy_package_path = _get_proxy_package_path_from_oras_response(response)
    _extract_proxy_tar_files(proxy_package_path, dest_dir, proxy_name)
    file_utils.delete_file(proxy_package_path, f"Failed to delete {proxy_package_path}. Please delete manually.", True)


def _get_proxy_package_path_from_oras_response(pull_response):
    if not isinstance(pull_response, list):
        raise AnsibleError(
            "Attempt to download Arc Connectivity Proxy returned unnexpected result. Please try again.")

    if len(pull_response) != 1:
        for r in pull_response:
            file_utils.delete_file(r, f"Failed to delete {r}. Please delete it manually.", True)
        raise AnsibleError(
            "Attempt to download Arc Connectivity Proxy returned unnexpected result. Please try again.")

    proxy_package_path = pull_response[0]

    if not os.path.isfile(proxy_package_path):
        raise AnsibleError("Unable to download Arc Connectivity Proxy. Please try again.")

    return proxy_package_path


def _extract_proxy_tar_files(proxy_package_path, install_dir, proxy_name):
    with tarfile.open(proxy_package_path, 'r:gz') as tar:
        members = []
        for member in tar.getmembers():
            if member.isfile():
                filenames = member.name.split('/')

                if len(filenames) != 2:
                    tar.close()
                    file_utils.delete_file(
                        proxy_package_path,
                        f"Failed to delete {proxy_package_path}. Please delete it manually.",
                        True)
                    raise AnsibleError(
                        "Attempt to download Arc Connectivity Proxy returned unnexpected result. Please try again.")

                member.name = filenames[1]

                if member.name.startswith('sshproxy'):
                    member.name = proxy_name
                elif member.name.lower() not in ['license.txt', 'thirdpartynotice.txt']:
                    tar.close()
                    file_utils.delete_file(
                        proxy_package_path,
                        f"Failed to delete {proxy_package_path}. Please delete it manually.",
                        True)
                    raise AnsibleError(
                        "Attempt to download Arc Connectivity Proxy returned unnexpected result. Please try again.")

                members.append(member)

        tar.extractall(members=members, path=install_dir)


def _check_proxy_installation(install_dir, proxy_name):
    proxy_filepath = os.path.join(install_dir, proxy_name)
    if not os.path.isfile(proxy_filepath):
        raise AnsibleError(
            "Failed to install required SSH Arc Connectivity Proxy. "
            f"Couldn't find expected file {proxy_filepath}. Please try again.")


def _get_proxy_filename(operating_system, architecture):
    if operating_system.lower() == 'darwin' and architecture == '386':
        raise AnsibleError("Unsupported Darwin OS with 386 architecture.")
    proxy_filename = \
        f"sshProxy_{operating_system.lower()}_{architecture}_{consts.CLIENT_PROXY_VERSION.replace('.', '_')}"
    if operating_system.lower() == 'windows':
        proxy_filename += '.exe'
    return proxy_filename


def _get_older_version_proxy_path(install_dir, operating_system, architecture):
    proxy_name = f"sshProxy_{operating_system.lower()}_{architecture}_*"
    return os.path.join(install_dir, proxy_name)


def _get_proxy_install_dir(arc_proxy_folder):
    if not arc_proxy_folder:
        return os.path.expanduser(os.path.join('~', ".clientsshproxy"))
    return arc_proxy_folder


def _get_client_architeture():
    import platform
    machine = platform.machine()
    architecture = None

    if "arm64" in machine.lower() or "aarch64" in machine.lower():
        architecture = 'arm64'
    elif machine.endswith('64'):
        architecture = 'amd64'
    elif machine.endswith('86'):
        architecture = '386'
    elif machine == '':
        raise AnsibleError("Couldn't identify the platform architecture.")
    else:
        raise AnsibleError(f"Unsuported architecture: {machine} is not currently supported")

    return architecture


def _get_client_operating_system():
    import platform
    operating_system = platform.system()

    if operating_system.lower() not in ('linux', 'darwin', 'windows'):
        raise AnsibleError(f"Unsuported OS: {operating_system} platform is not currently supported")
    return operating_system


def resource(client,
             url,
             method,
             body=None,
             custom_query=None,
             custom_header=None,
             status_code=None):

    # Construct status_code
    if status_code is None:
        status_code = [200, 201, 202]

    # Construct parameters
    query_parameters = {}
    query_parameters['api-version'] = '2023-03-15'
    if custom_query:
        query_parameters.update(custom_query)

    # Construct headers
    header_parameters = {}
    header_parameters['Content-Type'] = 'application/json; charset=utf-8'
    if custom_header:
        header_parameters.update(custom_header)

    response = client.query(url,
                            method,
                            query_parameters,
                            header_parameters,
                            body,
                            status_code,
                            600,
                            30)

    if hasattr(response, 'body'):
        try:
            response = json.loads(response.body())
        except Exception:
            response = response.body()
    elif hasattr(response, 'context'):
        response = response.context['deserialized_data']
    else:
        response = None

    return response



# Copyright (c) 2025, Dexter Le <dextersydney2001@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import re
import time
import typing as t

from ansible_collections.community.general.plugins.module_utils._cmd_runner import CmdRunner, cmd_runner_fmt

if t.TYPE_CHECKING:
    from ansible.module_utils.basic import AnsibleModule


_state_map = {
    "present": "create",
    "absent": "remove",
    "cloned": "clone",
    "status": "status",
    "enabled": "enable",
    "disabled": "disable",
    "online": "start",
    "offline": "stop",
    "maintenance": "set",
    "config": "config",
    "cleanup": "cleanup",
}


def fmt_resource_type(value):
    return [
        ":".join(
            value[k] for k in ["resource_standard", "resource_provider", "resource_name"] if value.get(k) is not None
        )
    ]


def fmt_resource_operation(value):
    cmd = []
    for op in value:
        cmd.append("op")
        cmd.append(op.get("operation_action"))
        for operation_option in op.get("operation_option"):
            cmd.append(operation_option)

    return cmd


def fmt_resource_argument(value):
    return ["--group" if value["argument_action"] == "group" else value["argument_action"]] + value["argument_option"]


def get_pacemaker_maintenance_mode(runner: CmdRunner) -> bool:
    with runner("cli_action config") as ctx:
        rc, out, err = ctx.run(cli_action="property")
        maint_mode_re = re.compile(r"maintenance-mode.*true", re.IGNORECASE)
        maintenance_mode_output = [line for line in out.splitlines() if maint_mode_re.search(line)]
        return bool(maintenance_mode_output)


def wait_for_resource(runner: CmdRunner, cli_noun: str, name: str, wait: int, sleep_interval: int = 5) -> None:
    """Poll ``pcs <cli_noun> status <name>`` until the resource reports Started or the wait budget expires.

    Raises an exception if the resource does not reach the Started state within *wait* seconds.
    """
    deadline = time.monotonic() + wait
    while True:
        with runner("cli_action state name") as ctx:
            rc, out, err = ctx.run(cli_action=cli_noun, state="status")
        if out and "Started" in out:
            return
        if time.monotonic() >= deadline:
            raise Exception(f"Timed out waiting {wait}s for {cli_noun} resource '{name}' to start")
        time.sleep(sleep_interval)


def pacemaker_runner(module: AnsibleModule, **kwargs) -> CmdRunner:
    runner_command = ["pcs"]
    runner = CmdRunner(
        module,
        command=runner_command,
        arg_formats=dict(
            cli_action=cmd_runner_fmt.as_list(),
            state=cmd_runner_fmt.as_map(_state_map),
            name=cmd_runner_fmt.as_list(),
            resource_type=cmd_runner_fmt.as_func(fmt_resource_type),
            resource_option=cmd_runner_fmt.as_list(),
            resource_operation=cmd_runner_fmt.as_func(fmt_resource_operation),
            resource_meta=cmd_runner_fmt.stack(cmd_runner_fmt.as_opt_val)("meta"),
            resource_argument=cmd_runner_fmt.as_func(fmt_resource_argument),
            resource_clone_ids=cmd_runner_fmt.as_list(),
            resource_clone_meta=cmd_runner_fmt.as_list(),
            apply_all=cmd_runner_fmt.as_bool("--all"),
            agent_validation=cmd_runner_fmt.as_bool("--agent-validation"),
            wait=cmd_runner_fmt.as_opt_eq_val("--wait"),
            config=cmd_runner_fmt.as_fixed("config"),
            force=cmd_runner_fmt.as_bool("--force"),
            version=cmd_runner_fmt.as_fixed("--version"),
            output_format=cmd_runner_fmt.as_opt_eq_val("--output-format"),
        ),
        **kwargs,
    )
    return runner



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_devices_system_memory_usage_history_by_interval_info
short_description: Information module for organizations _devices _system _memory _usage
  _history _by _interval
description:
  - Get all organizations _devices _system _memory _usage _history _by _interval.
  - Return the memory utilization history in kB for devices in the organization.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
  - cisco.meraki.module_info_pagination
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - OrganizationId path parameter. Organization ID.
    type: str
  perPage:
    description:
      - PerPage query parameter. The number of entries per page returned. Acceptable
        range is 3 - 20. Default is 10.
    type: int
  startingAfter:
    description:
      - >
        StartingAfter query parameter. A token used by the server to indicate the
        start of the page. Often this is a timestamp or an ID but it is not limited
        to those. This parameter should not be defined by client applications. The
        link for the first, last, prev, or next page in the HTTP Link header should
        define it.
    type: str
  endingBefore:
    description:
      - >
        EndingBefore query parameter. A token used by the server to indicate the end
        of the page. Often this is a timestamp or an ID but it is not limited to those.
        This parameter should not be defined by client applications. The link for
        the first, last, prev, or next page in the HTTP Link header should define
        it.
    type: str
  t0:
    description:
      - T0 query parameter. The beginning of the timespan for the data. The maximum
        lookback period is 31 days from today.
    type: str
  t1:
    description:
      - T1 query parameter. The end of the timespan for the data. T1 can be a maximum
        of 31 days after t0.
    type: str
  timespan:
    description:
      - >
        Timespan query parameter. The timespan for which the information will be fetched.
        If specifying timespan, do not specify parameters t0 and t1. The value must
        be in seconds and be less than or equal to 31 days. The default is 2 hours.
        If interval is provided, the timespan will be autocalculated.
    type: float
  interval:
    description:
      - >
        Interval query parameter. The time interval in seconds for returned data.
        The valid intervals are 300, 1200, 3600, 14400. The default is 300. Interval
        is calculated if time params are provided.
    type: int
  networkIds:
    description:
      - NetworkIds query parameter. Optional parameter to filter the result set by
        the included set of network IDs.
    elements: str
    type: list
  serials:
    description:
      - Serials query parameter. Optional parameter to filter device availabilities
        history by device serial numbers.
    elements: str
    type: list
  productTypes:
    description:
      - >
        ProductTypes query parameter. Optional parameter to filter device statuses
        by product type. Valid types are wireless, appliance, switch, systemsManager,
        camera, cellularGateway, sensor, wirelessController, campusGateway, and secureConnect.
    elements: str
    type: list
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations getOrganizationDevicesSystemMemoryUsageHistoryByInterval
    description: Complete reference of the getOrganizationDevicesSystemMemoryUsageHistoryByInterval
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-devices-system-memory-usage-history-by-interval
notes:
  - SDK Method used are
    organizations.Organizations.get_organization_devices_system_memory_usage_history_by_interval,
  - Paths used are
    get /organizations/{organizationId}/devices/system/memory/usage/history/byInterval,
"""

EXAMPLES = r"""
- name: Get all organizations _devices _system _memory _usage _history _by _interval
  cisco.meraki.organizations_devices_system_memory_usage_history_by_interval_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    perPage: 0
    startingAfter: string
    endingBefore: string
    t0: string
    t1: string
    timespan: 0
    interval: 0
    networkIds: []
    serials: []
    productTypes: []
    organizationId: string
    total_pages: -1
    direction: next
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "items": [
        {
          "serial": "string",
          "model": "string",
          "name": "string",
          "mac": "string",
          "tags": [
            "string"
          ],
          "provisioned": 0,
          "used": {
            "median": 0
          },
          "free": {
            "median": 0
          },
          "network": {
            "id": "string",
            "name": "string",
            "tags": [
              "string"
            ]
          },
          "intervals": [
            {
              "startTs": "string",
              "endTs": "string",
              "memory": {
                "used": {
                  "minimum": 0,
                  "maximum": 0,
                  "median": 0,
                  "percentages": {
                    "maximum": 0
                  }
                },
                "free": {
                  "minimum": 0,
                  "maximum": 0,
                  "median": 0
                }
              }
            }
          ]
        }
      ],
      "meta": {
        "counts": {
          "items": {
            "total": 0,
            "remaining": 0
          }
        }
      }
    }
"""



# Copyright (c) 2019, Sandeep Kasargod <sandeep@vexata.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this doc fragment is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations


class ModuleDocFragment:
    # Documentation fragment for Vexata VX100 series
    VX100 = r"""
options:
  array:
    description:
      - Vexata VX100 array hostname or IPv4 Address.
    required: true
    type: str
  user:
    description:
      - Vexata API user with administrative privileges.
      - Uses the E(VEXATA_USER) environment variable as a fallback.
    required: false
    type: str
  password:
    description:
      - Vexata API user password.
      - Uses the E(VEXATA_PASSWORD) environment variable as a fallback.
    required: false
    type: str
  validate_certs:
    description:
      - Allows connection when SSL certificates are not valid. Set to V(false) when certificates are not trusted.
      - If set to V(true), please make sure Python >= 2.7.9 is installed on the given machine.
    required: false
    type: bool
    default: false

requirements:
  - Vexata VX100 storage array with VXOS >= v3.5.0 on storage array
  - vexatapi >= 0.0.1
  - E(VEXATA_USER) and E(VEXATA_PASSWORD) environment variables must be set if
    user and password arguments are not passed to the module directly.
"""



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The Bgp_neighbor_address_family parser templates file. This contains
a list of parser definitions and associated functions that
facilitates both facts gathering and native command generation for
the given network resource.
"""

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)


def _tmplt_maximum_prefix(data):
    data = data["maximum_prefix"]
    cmd = "maximum-prefix {max_prefix_limit}".format(**data)
    if "generate_warning_threshold" in data:
        cmd += " {generate_warning_threshold}".format(**data)
    if "restart_interval" in data:
        cmd += " restart {restart_interval}".format(**data)
    if data.get("warning_only"):
        cmd += " warning-only"
    return cmd


class Bgp_neighbor_address_familyTemplate(NetworkTemplate):
    def __init__(self, lines=None):
        super(Bgp_neighbor_address_familyTemplate, self).__init__(lines=lines, tmplt=self)

    # fmt: off
    PARSERS = [
        {
            "name": "as_number",
            "getval": re.compile(
                r"""
                ^router\sbgp\s(?P<as_number>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "router bgp {{ as_number }}",
            "result": {
                "as_number": "{{ as_number }}",
            },
            "shared": True,
        },
        {
            "name": "address_family",
            "getval": re.compile(
                r"""
                (vrf\s(?P<vrf>\S+))?
                \s*neighbor\s(?P<neighbor>\S+)
                (\sremote-as\s\S+)?
                \saddress-family
                \s(?P<afi>\S+)
                (\s(?P<safi>\S+))?
                $""",
                re.VERBOSE,
            ),
            "setval": "address-family {{ afi }}{{ (' ' + safi) if safi is defined else '' }}",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{ neighbor }}": {
                                "neighbor_address": "{{ neighbor }}",
                                "address_family": {
                                    '{{ afi + "_" + safi|d() }}': {
                                        "afi": "{{ afi }}",
                                        "safi": "{{ safi }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
            "shared": True,
        },
        {
            "name": "advertise_map.exist_map",
            "getval": re.compile(
                r"""
                advertise-map
                \s(?P<route_map>\S+)
                \sexist-map\s(?P<exist_map>\S+)
                $""",
                re.VERBOSE,
            ),
            "setval": "advertise-map {{ advertise_map.route_map }} exist-map {{ advertise_map.exist_map }}",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{ neighbor }}": {
                                "address_family": {
                                    '{{ afi + "_" + safi|d() }}': {
                                        "advertise_map": {
                                            "route_map": "{{ route_map }}",
                                            "exist_map": "{{ exist_map }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "advertise_map.non_exist_map",
            "getval": re.compile(
                r"""
                advertise-map
                \s(?P<route_map>\S+)
                \snon-exist-map\s(?P<non_exist_map>\S+)
                $""",
                re.VERBOSE,
            ),
            "setval": "advertise-map {{ advertise_map.route_map }} non-exist-map {{ advertise_map.non_exist_map }}",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{ neighbor }}": {
                                "address_family": {
                                    '{{ afi + "_" + safi|d() }}': {
                                        "advertise_map": {
                                            "route_map": "{{ route_map }}",
                                            "non_exist_map": "{{ non_exist_map }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "advertisement_interval",
            "getval": re.compile(
                r"""
                advertisement-interval
                \s(?P<advertisement_interval>\d+)
                $""",
                re.VERBOSE,
            ),
            "setval": "advertisement-interval {{ advertisement_interval }}",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{ neighbor }}": {
                                "address_family": {
                                    '{{ afi + "_" + safi|d() }}': {
                                        "advertisement_interval": "{{ advertisement_interval }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "allowas_in",
            "getval": re.compile(
                r"""
                (?P<allowas_in>allowas-in)
                \s(?P<max_occurences>\d+)
                $""",
                re.VERBOSE,
            ),
            "setval": "allowas-in{{ ' ' + allowas_in.max_occurences|string if allowas_in.max_occurences is defined else '' }}",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{ neighbor }}": {
                                "address_family": {
                                    '{{ afi + "_" + safi|d() }}': {
                                        "allowas_in": {
                                            "set": "{{ True if allowas_in is defined and max_occurences is undefined }}",
                                            "max_occurences": "{{ max_occurences }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "as_override",
            "getval": re.compile(
                r"""
                (?P<as_override>as-override)
                $""",
                re.VERBOSE,
            ),
            "setval": "as-override",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{ neighbor }}": {
                                "address_family": {
                                    '{{ afi + "_" + safi|d() }}': {
                                        "as_override": "{{ not not as_override }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "capability.additional_paths.receive",
            "getval": re.compile(
                r"""
                capability\sadditional-paths
                \s(?P<receive>receive)
                (\s(?P<disable>disable))?
                $""",
                re.VERBOSE,
            ),
            "setval": "capability additional-paths receive{{ ' disable' if capability.additional_paths.receive == 'disable' else '' }}",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{ neighbor }}": {
                                "address_family": {
                                    '{{ afi + "_" + safi|d() }}': {
                                        "capability": {
                                            "additional_paths": {
                                                "receive": "{{ 'disable' if disable is defined else 'enable' }}",
                                            },
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "capability.additional_paths.send",
            "getval": re.compile(
                r"""
                capability\sadditional-paths
                \s(?P<send>send)
                (\s(?P<disable>disable))?
                $""",
                re.VERBOSE,
            ),
            "setval": "capability additional-paths send{{ ' disable' if capability.additional_paths.send == 'disable' else '' }}",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{ neighbor }}": {
                                "address_family": {
                                    '{{ afi + "_" + safi|d() }}': {
                                        "capability": {
                                            "additional_paths": {
                                                "send": "{{ 'disable' if disable is defined else 'enable' }}",
                                            },
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "default_originate",
            "getval": re.compile(
                r"""
                (?P<default_originate>default-originate)
                (\sroute-map\s(?P<route_map>\S+))?
                $""",
                re.VERBOSE,
            ),
            "setval": "default-originate{{ ' route-map ' + default_originate.route_map if default_originate.route_map is defined else '' }}",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{ neighbor }}": {
                                "address_family": {
                                    '{{ afi + "_" + safi|d() }}': {
                                        "default_originate": {
                                            "set": "{{ True if default_originate is defined and route_map is not defined }}",
                                            "route_map": "{{ route_map }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "disable_peer_as_check",
            "getval": re.compile(
                r"""
                (?P<disable_peer_as_check>disable-peer-as-check)
                $""",
                re.VERBOSE,
            ),
            "setval": "disable-peer-as-check",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{ neighbor }}": {
                                "address_family": {
                                    '{{ afi + "_" + safi|d() }}': {
                                        "disable_peer_as_check": "{{ not not disable_peer_as_check }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "filter_list.inbound",
            "getval": re.compile(
                r"""
                filter-list
                \s(?P<in>\S+)\s(?:in)
                $""",
                re.VERBOSE,
            ),
            "setval": "filter-list {{ filter_list.inbound }} in",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{ neighbor }}": {
                                "address_family": {
                                    '{{ afi + "_" + safi|d() }}': {
                                        "filter_list": {
                                            "inbound": "{{ in }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "filter_list.outbound",
            "getval": re.compile(
                r"""
                filter-list
                \s(?P<out>\S+)\s(?:out)
                $""",
                re.VERBOSE,
            ),
            "setval": "filter-list {{ filter_list.outbound }} out",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{ neighbor }}": {
                                "address_family": {
                                    '{{ afi + "_" + safi|d() }}': {
                                        "filter_list": {
                                            "outbound": "{{ out }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "inherit",
            "getval": re.compile(
                r"""
                inherit\speer-policy
                \s(?P<template>\S+)
                \s(?P<sequence>\d+)
                $""",
                re.VERBOSE,
            ),
            "setval": "inherit peer-policy {{ inherit.template }} {{ inherit.sequence }}",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{ neighbor }}": {
                                "address_family": {
                                    '{{ afi + "_" + safi|d() }}': {
                                        "inherit": {
                                            "template": "{{ template }}",
                                            "sequence": "{{ sequence }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "maximum_prefix",
            "getval": re.compile(
                r"""
                maximum-prefix
                \s(?P<max_prefix_limit>\d+)
                (\s(?P<generate_warning_threshold>\d+))?
                (\srestart\s(?P<restart_interval>\d+))?
                (\s(?P<warning_only>warning-only))?
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_maximum_prefix,
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{ neighbor }}": {
                                "address_family": {
                                    '{{ afi + "_" + safi|d() }}': {
                                        "maximum_prefix": {
                                            "max_prefix_limit": "{{ max_prefix_limit }}",
                                            "generate_warning_threshold": "{{ generate_warning_threshold }}",
                                            "restart_interval": "{{ restart_interval }}",
                                            "warning_only": "{{ not not warning_only }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "next_hop_self",
            "getval": re.compile(
                r"""
                (?P<next_hop_self>next-hop-self)
                (\s(?P<all_routes>all))?
                $""",
                re.VERBOSE,
            ),
            "setval": "next-hop-self{{ ' all' if next_hop_self.all_routes|d(False) else '' }}",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{ neighbor }}": {
                                "address_family": {
                                    '{{ afi + "_" + safi|d() }}': {
                                        "next_hop_self": {
                                            "set": "{{ True if next_hop_self is defined and all_routes is not defined }}",
                                            "all_routes": "{{ not not all_routes }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "next_hop_third_party",
            "getval": re.compile(
                r"""
                no\s(?P<next_hop_third_party>next-hop-third-party)
                $""",
                re.VERBOSE,
            ),
            "setval": "next-hop-third-party",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{ neighbor }}": {
                                "address_family": {
                                    '{{ afi + "_" + safi|d() }}': {
                                        "next_hop_third_party": "{{ not next_hop_third_party }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "prefix_list.inbound",
            "getval": re.compile(
                r"""
                prefix-list
                \s(?P<in>\S+)\s(?:in)
                $""",
                re.VERBOSE,
            ),
            "setval": "prefix-list {{ prefix_list.inbound }} in",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{ neighbor }}": {
                                "address_family": {
                                    '{{ afi + "_" + safi|d() }}': {
                                        "prefix_list": {
                                            "inbound": "{{ in }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "prefix_list.outbound",
            "getval": re.compile(
                r"""
                prefix-list
                \s(?P<out>\S+)\s(?:out)
                $""",
                re.VERBOSE,
            ),
            "setval": "prefix-list {{ prefix_list.outbound }} out",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{ neighbor }}": {
                                "address_family": {
                                    '{{ afi + "_" + safi|d() }}': {
                                        "prefix_list": {
                                            "outbound": "{{ out }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "rewrite_evpn_rt_asn",
            "getval": re.compile(
                r"""
                (?P<rewrite_evpn_rt_asn>rewrite-evpn-rt-asn)
                $""",
                re.VERBOSE,
            ),
            "setval": "rewrite-evpn-rt-asn",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{ neighbor }}": {
                                "address_family": {
                                    '{{ afi + "_" + safi|d() }}': {
                                        "rewrite_evpn_rt_asn": "{{ not not rewrite_evpn_rt_asn }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "rewrite_rt_asn",
            "getval": re.compile(
                r"""
                (?P<rewrite_rt_asn>rewrite-rt-asn)
                $""",
                re.VERBOSE,
            ),
            "setval": "rewrite-rt-asn",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{ neighbor }}": {
                                "address_family": {
                                    '{{ afi + "_" + safi|d() }}': {
                                        "rewrite_rt_asn": "{{ not not rewrite_rt_asn }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "route_map.inbound",
            "getval": re.compile(
                r"""
                route-map
                \s(?P<in>\S+)\s(?:in)
                $""",
                re.VERBOSE,
            ),
            "setval": "route-map {{ route_map.inbound }} in",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{ neighbor }}": {
                                "address_family": {
                                    '{{ afi + "_" + safi|d() }}': {
                                        "route_map": {
                                            "inbound": "{{ in }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "route_map.outbound",
            "getval": re.compile(
                r"""
                route-map
                \s(?P<out>\S+)\s(?:out)
                $""",
                re.VERBOSE,
            ),
            "setval": "route-map {{ route_map.outbound }} out",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{ neighbor }}": {
                                "address_family": {
                                    '{{ afi + "_" + safi|d() }}': {
                                        "route_map": {
                                            "outbound": "{{ out }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "route_reflector_client",
            "getval": re.compile(
                r"""
                (?P<route_reflector_client>route-reflector-client)
                $""",
                re.VERBOSE,
            ),
            "setval": "route-reflector-client",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{ neighbor }}": {
                                "address_family": {
                                    '{{ afi + "_" + safi|d() }}': {
                                        "route_reflector_client": "{{ not not route_reflector_client }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "send_community.standard",
            "getval": re.compile(
                r"""
                (?P<send_community>send-community)
                $""",
                re.VERBOSE,
            ),
            "setval": "send-community",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{ neighbor }}": {
                                "address_family": {
                                    '{{ afi + "_" + safi|d() }}': {
                                        "send_community": {
                                            "standard": "{{ True if send_community is defined }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "send_community.extended",
            "getval": re.compile(
                r"""
                send-community
                \s(?P<extended>extended)
                $""",
                re.VERBOSE,
            ),
            "setval": "send-community extended",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{ neighbor }}": {
                                "address_family": {
                                    '{{ afi + "_" + safi|d() }}': {
                                        "send_community": {
                                            "extended": "{{ True if extended is defined }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "soft_reconfiguration_inbound",
            "getval": re.compile(
                r"""
                (?P<soft_reconfiguration_inbound>soft-reconfiguration\sinbound)
                (\s(?P<always>always))?
                $""",
                re.VERBOSE,
            ),
            "setval": "soft-reconfiguration inbound{{ ' always' if soft_reconfiguration_inbound.always|d(False) else '' }}",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{ neighbor }}": {
                                "address_family": {
                                    '{{ afi + "_" + safi|d() }}': {
                                        "soft_reconfiguration_inbound": {
                                            "set": "{{ True if soft_reconfiguration_inbound is defined and always is undefined }}",
                                            "always": "{{ not not always }}",
                                        },
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "soo",
            "getval": re.compile(
                r"""
                soo\s(?P<soo>\S+)
                $""",
                re.VERBOSE,
            ),
            "setval": "soo {{ soo }}",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{ neighbor }}": {
                                "address_family": {
                                    '{{ afi + "_" + safi|d() }}': {
                                        "soo": "{{ soo }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "suppress_inactive",
            "getval": re.compile(
                r"""
                (?P<suppress_inactive>suppress-inactive)
                $""",
                re.VERBOSE,
            ),
            "setval": "suppress-inactive",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{ neighbor }}": {
                                "address_family": {
                                    '{{ afi + "_" + safi|d() }}': {
                                        "suppress_inactive": "{{ not not suppress_inactive }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "unsuppress_map",
            "getval": re.compile(
                r"""
                unsuppress-map\s(?P<unsuppress_map>\S+)
                $""",
                re.VERBOSE,
            ),
            "setval": "unsuppress-map {{ unsuppress_map }}",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{ neighbor }}": {
                                "address_family": {
                                    '{{ afi + "_" + safi|d() }}': {
                                        "unsuppress_map": "{{ unsuppress_map }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "weight",
            "getval": re.compile(
                r"""
                weight\s(?P<weight>\d+)
                $""",
                re.VERBOSE,
            ),
            "setval": "weight {{ weight }}",
            "result": {
                "vrfs": {
                    "{{ 'vrf_' + vrf|d() }}": {
                        "vrf": "{{ vrf }}",
                        "neighbors": {
                            "{{ neighbor }}": {
                                "address_family": {
                                    '{{ afi + "_" + safi|d() }}': {
                                        "weight": "{{ weight }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
    ]
    # fmt: on



#!/usr/bin/python
#
# Copyright (c) 2024, Florian Apolloner (@apollo13)
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: consul_auth_method
short_description: Manipulate Consul auth methods
version_added: 8.3.0
description:
  - Allows the addition, modification and deletion of auth methods in a Consul cluster using the agent. For more details on
    using and configuring ACLs, see U(https://www.consul.io/docs/guides/acl.html).
author:
  - Florian Apolloner (@apollo13)
extends_documentation_fragment:
  - community.general._consul
  - community.general._consul.actiongroup_consul
  - community.general._consul.token
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: partial
    details:
      - In check mode the diff misses operational attributes.
options:
  state:
    description:
      - Whether the token should be present or absent.
    choices: ['present', 'absent']
    default: present
    type: str
  name:
    description:
      - Specifies a name for the ACL auth method.
      - The name can contain alphanumeric characters, dashes C(-), and underscores C(_).
    type: str
    required: true
  type:
    description:
      - The type of auth method being configured.
      - This field is immutable.
      - Required when the auth method is created.
    type: str
    choices: ['kubernetes', 'jwt', 'oidc', 'aws-iam']
  description:
    description:
      - Free form human readable description of the auth method.
    type: str
  display_name:
    description:
      - An optional name to use instead of O(name) when displaying information about this auth method.
    type: str
  max_token_ttl:
    description:
      - This specifies the maximum life of any token created by this auth method.
      - Can be specified in the form of V(60s) or V(5m) (that is, 60 seconds or 5 minutes, respectively).
    type: str
  token_locality:
    description:
      - Defines the kind of token that this auth method should produce.
    type: str
    choices: ['local', 'global']
  config:
    description:
      - The raw configuration to use for the chosen auth method.
      - Contents vary depending upon the O(type) chosen.
      - Required when the auth method is created.
    type: dict
"""

EXAMPLES = r"""
- name: Create an auth method
  community.general.consul_auth_method:
    name: test
    type: jwt
    config:
      jwt_validation_pubkeys:
        - |
          -----BEGIN PUBLIC KEY-----
          MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAu1SU1LfVLPHCozMxH2Mo
          4lgOEePzNm0tRgeLezV6ffAt0gunVTLw7onLRnrq0/IzW7yWR7QkrmBL7jTKEn5u
          +qKhbwKfBstIs+bMY2Zkp18gnTxKLxoS2tFczGkPLPgizskuemMghRniWaoLcyeh
          kd3qqGElvW/VDL5AaWTg0nLVkjRo9z+40RQzuVaE8AkAFmxZzow3x+VJYKdjykkJ
          0iT9wCS0DRTXu269V264Vf/3jvredZiKRkgwlL9xNAwxXFg0x/XFw005UWVRIkdg
          cKWTjpBP2dPwVZ4WWC+9aGVd+Gyn1o0CLelf4rEjGoXbAAEgAqeGUxrcIlbjXfbc
          mwIDAQAB
          -----END PUBLIC KEY-----
    token: "{{ consul_management_token }}"

- name: Delete auth method
  community.general.consul_auth_method:
    name: test
    state: absent
    token: "{{ consul_management_token }}"
"""

RETURN = r"""
auth_method:
  description: The auth method as returned by the Consul HTTP API.
  returned: always
  type: dict
  sample:
    Config:
      JWTValidationPubkeys:
        - |-
          -----BEGIN PUBLIC KEY-----
          MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAu1SU1LfVLPHCozMxH2Mo
          4lgOEePzNm0tRgeLezV6ffAt0gunVTLw7onLRnrq0/IzW7yWR7QkrmBL7jTKEn5u
          +qKhbwKfBstIs+bMY2Zkp18gnTxKLxoS2tFczGkPLPgizskuemMghRniWaoLcyeh
          kd3qqGElvW/VDL5AaWTg0nLVkjRo9z+40RQzuVaE8AkAFmxZzow3x+VJYKdjykkJ
          0iT9wCS0DRTXu269V264Vf/3jvredZiKRkgwlL9xNAwxXFg0x/XFw005UWVRIkdg
          cKWTjpBP2dPwVZ4WWC+9aGVd+Gyn1o0CLelf4rEjGoXbAAEgAqeGUxrcIlbjXfbc
          mwIDAQAB
          -----END PUBLIC KEY-----
    CreateIndex: 416
    ModifyIndex: 487
    Name: test
    Type: jwt
operation:
  description: The operation performed.
  returned: changed
  type: str
  sample: update
"""

import re

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._consul import (
    AUTH_ARGUMENTS_SPEC,
    _ConsulModule,
    camel_case_key,
)


def normalize_ttl(ttl):
    matches = re.findall(r"(\d+)(:h|m|s)", ttl)
    ttl = 0
    for value, unit in matches:
        value = int(value)
        if unit == "m":
            value *= 60
        elif unit == "h":
            value *= 60 * 60
        ttl += value

    new_ttl = ""
    hours, remainder = divmod(ttl, 3600)
    if hours:
        new_ttl += f"{hours}h"
    minutes, seconds = divmod(remainder, 60)
    if minutes:
        new_ttl += f"{minutes}m"
    if seconds:
        new_ttl += f"{seconds}s"
    return new_ttl


class ConsulAuthMethodModule(_ConsulModule):
    api_endpoint = "acl/auth-method"
    result_key = "auth_method"
    unique_identifiers = ["name"]

    def map_param(self, k, v, is_update):
        if k == "config" and v:
            v = {camel_case_key(k2): v2 for k2, v2 in v.items()}
        return super().map_param(k, v, is_update)

    def needs_update(self, api_obj, module_obj):
        if "MaxTokenTTL" in module_obj:
            module_obj["MaxTokenTTL"] = normalize_ttl(module_obj["MaxTokenTTL"])
        return super().needs_update(api_obj, module_obj)


_ARGUMENT_SPEC = {
    "name": dict(type="str", required=True),
    "type": dict(type="str", choices=["kubernetes", "jwt", "oidc", "aws-iam"]),
    "description": dict(type="str"),
    "display_name": dict(type="str"),
    "max_token_ttl": dict(type="str", no_log=False),
    "token_locality": dict(type="str", choices=["local", "global"]),
    "config": dict(type="dict"),
    "state": dict(default="present", choices=["present", "absent"]),
}
_ARGUMENT_SPEC.update(AUTH_ARGUMENTS_SPEC)


def main():
    module = AnsibleModule(
        _ARGUMENT_SPEC,
        supports_check_mode=True,
    )
    consul_module = ConsulAuthMethodModule(module)
    consul_module.execute()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2024, Akini Ross (@akinross) <akinross@cisco.com>

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: ndo_dhcp_relay_policy
short_description: Manage DHCP Relay Policies on Cisco Nexus Dashboard Orchestrator (NDO).
description:
- Manage DHCP Relay Policies on Cisco Nexus Dashboard Orchestrator (NDO).
- This module is only supported on ND v3.1 (NDO v4.3) and later.
author:
- Akini Ross (@akinross)
options:
  template:
    description:
    - The name of the template.
    - The template must be a tenant template.
    type: str
    required: true
  relay_policy:
    description:
    - The name of the DHCP Relay Policy.
    type: str
    aliases: [ name ]
  relay_policy_uuid:
    description:
    - The uuid of the DHCP Relay Policy.
    - This parameter is required when the O(relay_policy) needs to be updated.
    type: str
    aliases: [ uuid ]
  description:
    description:
    - The description of the DHCP Relay Policy.
    type: str
  providers:
    description:
    - A list of providers attached to the DHCP Relay Policy.
    - The list of configured providers must contain at least one provider.
    - When the list of providers is null the update will not change existing providers configuration.
    type: list
    elements: dict
    suboptions:
      schema:
        description:
        - The name of the schema for the (External) EPG.
        type: str
        required: true
      template:
        description:
        - The name of the template for the (External) EPG.
        type: str
        required: true
      anp:
        description:
        - The name of the Application Profile (ANP).
        - This parameter is required when O(providers.epg) is provided.
        type: str
      epg:
        description:
        - The name of the Endpoint Group (EPG).
        - O(providers.epg) and O(providers.external_epg) are mutually exclusive.
        - O(providers.epg) is required when O(providers.external_epg) is not provided.
        type: str
      external_epg:
        description:
        - The name of the External Endpoint Group (EPG).
        - O(providers.external_epg) and O(providers.epg) are mutually exclusive.
        - O(providers.external_epg) is required when O(providers.epg) is not provided.
        type: str
      ip:
        description:
        - The IP address of the DHCP server.
        type: str
        required: true
      use_server_vrf:
        description:
        - Use the server VRF.
        type: bool
        default: false
  state:
    description:
    - Use C(absent) for removing.
    - Use C(query) for listing an object or multiple objects.
    - Use C(present) for creating or updating.
    type: str
    choices: [ absent, query, present ]
    default: query
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Create a new dhcp relay policy
  cisco.mso.ndo_dhcp_relay_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    relay_policy: ansible_test_relay_policy
    providers:
      - schema: ansible_test_schema
        template: ansible_test_template
        anp: ansible_test_anp
        epg: ansible_test_epg_1
        ip: 1.1.1.1
    state: present
  register: create

- name: Query a dhcp relay policy with name
  cisco.mso.ndo_dhcp_relay_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    relay_policy: ansible_test_relay_policy
    state: query
  register: query_one

- name: Query a dhcp relay policy with UUID
  cisco.mso.ndo_dhcp_relay_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    relay_policy_uuid: '{{ create.current.uuid }}'
    state: query
  register: query_with_uuid

- name: Query all dhcp relay policy in the template
  cisco.mso.ndo_dhcp_relay_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    state: query
  register: query_all

- name: Delete a dhcp relay policy
  cisco.mso.ndo_dhcp_relay_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    relay_policy: ansible_test_relay_policy
    state: absent
"""

RETURN = r"""
"""


import copy
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec
from ansible_collections.cisco.mso.plugins.module_utils.schema import MSOSchema
from ansible_collections.cisco.mso.plugins.module_utils.template import MSOTemplate, KVPair


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        template=dict(type="str", required=True),
        relay_policy=dict(type="str", aliases=["name"]),
        relay_policy_uuid=dict(type="str", aliases=["uuid"]),
        description=dict(type="str"),
        providers=dict(
            type="list",
            elements="dict",
            options=dict(
                schema=dict(type="str", required=True),
                template=dict(type="str", required=True),
                anp=dict(type="str"),
                epg=dict(type="str"),
                external_epg=dict(type="str"),
                ip=dict(type="str", required=True),
                use_server_vrf=dict(type="bool", default=False),
            ),
        ),
        state=dict(type="str", default="query", choices=["absent", "query", "present"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["relay_policy"]],
            ["state", "present", ["relay_policy"]],
        ],
    )

    mso = MSOModule(module)

    template = module.params.get("template")
    relay_policy = module.params.get("relay_policy")
    relay_policy_uuid = module.params.get("relay_policy_uuid")
    providers = get_providers_payload(mso, module.params.get("providers")) if module.params.get("providers") else []
    description = module.params.get("description")
    state = module.params.get("state")

    ops = []
    match = None
    err_message_min_providers = "At least one provider is required when state is present."

    mso_template = MSOTemplate(mso, "tenant", template)
    mso_template.validate_template("tenantPolicy")

    path = "/tenantPolicyTemplate/template/dhcpRelayPolicies"
    match = get_dhcp_relay_policy(mso_template, relay_policy_uuid, relay_policy)

    if relay_policy_uuid or relay_policy:
        if match:
            mso.existing = mso.previous = copy.deepcopy(insert_dhcp_relay_policy_relation_name(match.details, mso_template))  # Query a specific object
    elif match:
        mso.existing = [insert_dhcp_relay_policy_relation_name(dhcp_relay_policy, mso_template) for dhcp_relay_policy in match]  # Query all objects

    if state == "present":
        if match:
            if module.params.get("providers") is not None and len(providers) == 0:
                mso.fail_json(msg=err_message_min_providers)

            if relay_policy and match.details.get("name") != relay_policy:
                ops.append(dict(op="replace", path="{0}/{1}/name".format(path, match.index), value=relay_policy))
                match.details["name"] = relay_policy

            if description is not None and match.details.get("description") != description:
                ops.append(dict(op="replace", path="{0}/{1}/description".format(path, match.index), value=description))
                match.details["description"] = description

            if module.params.get("providers") is not None and match.details.get("providers") != providers:
                ops.append(dict(op="replace", path="{0}/{1}/providers".format(path, match.index), value=providers))
                match.details["providers"] = providers

            mso.sanitize(match.details)

        else:
            if not providers:
                mso.fail_json(msg=err_message_min_providers)

            payload = {"name": relay_policy, "providers": providers}
            if description:
                payload["description"] = description

            ops.append(dict(op="add", path="{0}/-".format(path), value=payload))

            mso.sanitize(payload)

        mso.existing = mso.proposed

    elif state == "absent":
        if match:
            ops.append(dict(op="remove", path="{0}/{1}".format(path, match.index)))
        mso.existing = {}

    if not module.check_mode and ops:
        mso_template.template = mso.request(mso_template.template_path, method="PATCH", data=ops)
        match = get_dhcp_relay_policy(mso_template, relay_policy_uuid, relay_policy)
        if match:
            mso.existing = insert_dhcp_relay_policy_relation_name(match.details, mso_template)  # When the state is present
        else:
            mso.existing = {}  # When the state is absent
    elif module.check_mode and state != "query":  # When the state is present/absent with check mode
        mso.existing = mso.proposed if state == "present" else {}

    mso.exit_json()


def insert_dhcp_relay_policy_relation_name(dhcp_relay_policy, mso_template):
    for provider in dhcp_relay_policy.get("providers"):
        if provider.get("epgRef"):
            provider["epgName"] = mso_template.get_template_object_name_by_uuid("epg", provider.get("epgRef"))
        if provider.get("externalEpgRef"):
            provider["externalEpgName"] = mso_template.get_template_object_name_by_uuid("externalEpg", provider.get("externalEpgRef"))
    return dhcp_relay_policy


def get_dhcp_relay_policy(mso_template, uuid=None, name=None, fail_module=False):
    match = mso_template.template.get("tenantPolicyTemplate", {}).get("template", {}).get("dhcpRelayPolicies", [])
    if uuid or name:  # Query a specific object
        return mso_template.get_object_by_key_value_pairs("DHCP Relay Policy", match, [KVPair("uuid", uuid) if uuid else KVPair("name", name)], fail_module)
    return match  # Query all objects


def get_providers_payload(mso, providers):
    # Cache used to reduce the number of schema queries done by MSOSchema function.
    schema_cache = {}

    payload = []
    for provider in providers:
        schema = provider.get("schema")
        template = provider.get("template")
        anp = provider.get("anp")
        epg = provider.get("epg")
        external_epg = provider.get("external_epg")
        ip = provider.get("ip")

        provider_payload = {"ip": ip, "useServerVrf": provider.get("use_server_vrf")}

        # Check if schema is already in cache, if not create a new MSOSchema object and add it to the cache.
        if not schema_cache.get(schema):
            schema_cache[schema] = MSOSchema(mso, schema)

        schema_cache[schema].set_template(template)

        if epg and not anp:
            mso.fail_json(msg="The anp argument is required for each provider when the epg argument is provided.")
        elif not epg and not external_epg:
            mso.fail_json(msg="The epg or external_epg argument is required for each provider.")
        elif epg and external_epg:
            mso.fail_json(msg="The epg and external_epg arguments are mutually exclusive for each provider.")
        elif external_epg and anp:
            mso.fail_json(msg="The anp and external_epg arguments are mutually exclusive for each provider.")
        elif epg:
            schema_cache[schema].set_template_anp(anp)
            schema_cache[schema].set_template_anp_epg(epg)
            provider_payload["epgRef"] = schema_cache[schema].schema_objects["template_anp_epg"].details.get("uuid")
            provider_payload["epgName"] = epg
        else:
            schema_cache[schema].set_template_external_epg(external_epg)
            provider_payload["externalEpgRef"] = schema_cache[schema].schema_objects.get("template_external_epg").details.get("uuid")
            provider_payload["externalEpgName"] = external_epg

        payload.append(provider_payload)
    return payload


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2021 Sebastian Gruber ,dacoso GmbH All Rights Reserved.
# Copyright: (c) 2019, Hitachi ID Systems, Inc.
# SPDX-License-Identifier: GPL-3.0-only
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r'''
---
module: win_dns_record
short_description: Manage Windows Server DNS records
description:
- Manage DNS records within an existing Windows Server DNS zone.
version_added: 2.6.0
author:
 - Sebastian Gruber (@sgruber94)
 - John Nelson (@johnboy2)
requirements:
  - This module requires Windows 8, Server 2012, or newer.
options:
  name:
    description:
    - The name of the record.
    required: yes
    type: str
  port:
    description:
    - The port number of the record.
    - Required when C(type=SRV).
    - Supported only for C(type=SRV).
    type: int
  priority:
    description:
    - The priority number for each service in SRV record.
    - Required when C(type=SRV).
    - Supported only for C(type=SRV).
    type: int
  state:
    description:
    - Whether the record should exist or not.
    choices: [ absent, present ]
    default: present
    type: str
  ttl:
    description:
    - The "time to live" of the record, in seconds.
    - Ignored when C(state=absent).
    - Valid range is 1 - 31557600.
    - Note that an Active Directory forest can specify a minimum TTL, and will
      dynamically "round up" other values to that minimum.
    default: 3600
    type: int
  aging:
    description:
    - Should aging be activated for the record.
    - If set to C(false), the record will be static.
    default: false
    type: bool
  allow_update_any:
    description:
    - Specifies if any authenticated user can update the record.
    - If set to C(true), any authenticated user can update the record.
    - If set to C(false), only the account that created the record or accounts
      with appropriate permissions can update it.
    - For non-SRV records, this was previously hardcoded to C(true). Since C(ansible.windows>=3.5.0) it now
      defaults to C(false) to ensure records are secure by default.
    default: false
    type: bool
    version_added: '3.5.0'
  type:
    description:
    - The type of DNS record to manage.
    - C(SRV) was added in the 1.0.0 release of this collection.
    - C(NS) was added in the 1.1.0 release of this collection.
    - C(TXT) was added in the 1.6.0 release of this collection.
    - C(DHCID) was added in the 1.12.0 release of this collection.
    choices: [ A, AAAA, CNAME, DHCID, NS, PTR, SRV, TXT ]
    required: yes
    type: str
  value:
    description:
    - The value(s) to specify. Required when C(state=present).
    - When C(type=PTR) only the partial part of the IP should be given.
    - Multiple values can be passed when C(type=NS)
    aliases: [ values ]
    default: []
    type: list
    elements: str
  weight:
    description:
    - Weightage given to each service record in SRV record.
    - Required when C(type=SRV).
    - Supported only for C(type=SRV).
    type: int
  zone:
    description:
    - The name of the zone to manage (eg C(example.com)).
    - The zone must already exist.
    required: yes
    type: str
  zone_scope:
    description:
    - The name of the zone scope to manage (eg C(ScopeAZ)).
    - The zone must already exist.
    required: no
    type: str
  computer_name:
    description:
      - Specifies a DNS server.
      - You can specify an IP address or any value that resolves to an IP
        address, such as a fully qualified domain name (FQDN), host name, or
        NETBIOS name.
    type: str
'''

EXAMPLES = r'''
# Demonstrate creating a matching A and PTR record.

- name: Create database server record
  ansible.windows.win_dns_record:
    name: "cgyl1404p"
    type: "A"
    value: "10.1.1.1"
    zone: "amer.example.com"

- name: Create matching PTR record
  ansible.windows.win_dns_record:
    name: "1.1.1"
    type: "PTR"
    value: "db1"
    zone: "10.in-addr.arpa"

# Demonstrate replacing an A record with a CNAME

- name: Remove static record
  ansible.windows.win_dns_record:
    name: "db1"
    type: "A"
    state: absent
    zone: "amer.example.com"

- name: Create database server alias
  ansible.windows.win_dns_record:
    name: "db1"
    type: "CNAME"
    value: "cgyl1404p.amer.example.com"
    zone: "amer.example.com"

# Demonstrate creating multiple A records for the same name

- name: Create multiple A record values for www
  ansible.windows.win_dns_record:
    name: "www"
    type: "A"
    values:
      - 10.0.42.5
      - 10.0.42.6
      - 10.0.42.7
    zone: "example.com"

# Demonstrates a partial update (replace some existing values with new ones)
# for a pre-existing name

- name: Update www host with new addresses
  ansible.windows.win_dns_record:
    name: "www"
    type: "A"
    values:
      - 10.0.42.5  # this old value was kept (others removed)
      - 10.0.42.12  # this new value was added
    zone: "example.com"

# Demonstrate creating a SRV record

- name: Creating a SRV record with port number and priority
  ansible.windows.win_dns_record:
    name: "test"
    priority: 5
    port: 995
    state: present
    type: "SRV"
    weight: 2
    value: "amer.example.com"
    zone: "example.com"

# Demonstrate creating a NS record with multiple values

- name: Creating NS record
  ansible.windows.win_dns_record:
    name: "ansible.prog"
    state: present
    type: "NS"
    values:
      - 10.0.0.1
      - 10.0.0.2
      - 10.0.0.3
      - 10.0.0.4
    zone: "example.com"

# Demonstrate creating a TXT record

- name: Creating a TXT record with descriptive Text
  ansible.windows.win_dns_record:
    name: "test"
    state: present
    type: "TXT"
    value: "justavalue"
    zone: "example.com"

# Demostrate creating a A record to Zone Scope

- name: Create database server record
  ansible.windows.win_dns_record:
    name: "cgyl1404p.amer.example.com"
    type: "A"
    value: "10.1.1.1"
    zone: "amer.example.com"
    zone_scope: "external"
'''

RETURN = r'''
'''



#!/usr/bin/python
#
# Copyright (c) 2023 xuzhang3 (@xuzhang3), Fred-sun (@Fred-sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_sshpublickey
version_added: "2.0.0"
short_description: Manage ssh public key with vm
description:
    - Create, update or delete the vm public key.
options:
    resource_group:
        description:
            - Name of resource group.
        required: true
        type: str
    location:
        description:
            - Valid Azure location. Defaults to location of the resource group.
        type: str
    name:
        description:
            - The name of the SSH public key.
        required: true
        type: str
    public_key:
        description:
            - SSH public key used to authenticate to a virtual machine through ssh.
            - If this property is not initially provided when the resource is created, the publicKey property will be populated when generateKeyPair is called.
            - If the public key is provided upon resource creation, the provided public key needs to be at least 2048-bit and in ssh-rsa format.
        type: str
    state:
        description:
            - State of the SSH Public Key. Use C(present) to create or update and C(absent) to delete.
        default: present
        type: str
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - xuzhang3 (@xuzhang3)
    - Fred-sun (@Fred-sun)

'''

EXAMPLES = '''
- name: Create a SSH Public Key
  azure_rm_sshpublickey:
    resource_group: myResourceGroup
    name: mySshPublicKey
    public_key: "ssh-rsa ****************************@test.com"
    tags:
      testing: testing
      delete: on-exit

- name: Generate a pair SSH Public Key
  azure_rm_sshpublickey:
    resource_group: myResourceGroup
    name: mySshPublicKey
    tags:
      testing: testing
      delete: on-exit

- name: Delete a SSH Public Key
  azure_rm_sshpublickey:
    resource_group: myResourceGroup
    name: mySshPublicKey
    state: absent
'''
RETURN = '''
state:
    description:
        - Current state of the SSH Public Key.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Resource ID.
            returned: always
            type: str
            sample: /subscriptions/xxxx/resourceGroups/xxx/providers/Microsoft.Compute/sshPublicKeys/mySshPublicKeyName
        location:
            description:
                - The Geo-location where the resource lives.
            returned: always
            type: str
            sample: eastus
        name:
            description:
                - Resource name.
            returned: always
            type: str
            sample: mySshPublicKey
        tags:
            description:
                - Resource tags, such as { 'tags1':'value1' }.
            returned: always
            type: dict
            sample: { 'key1':'value1' }
        public_key:
            description:
                - SSH public key used to authenticate to a virtual machine through ssh.
            returned: always
            type: str
            sample: "ssh-rsa **************@test.com"
'''

try:
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase


class AzureRMSshPublicKey(AzureRMModuleBase):

    def __init__(self):

        self.module_arg_spec = dict(
            resource_group=dict(type='str', required=True),
            name=dict(type='str', required=True),
            state=dict(type='str', default='present', choices=['present', 'absent']),
            location=dict(type='str'),
            public_key=dict(type='str'),
        )

        self.resource_group = None
        self.name = None
        self.state = None
        self.location = None
        self.public_key = None

        self.body = dict()

        self.results = dict(
            changed=False,
            state=dict()
        )

        super(AzureRMSshPublicKey, self).__init__(self.module_arg_spec,
                                                  supports_tags=True,
                                                  supports_check_mode=True)

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            setattr(self, key, kwargs[key])
            for key in ['tags', 'public_key']:
                self.body[key] = kwargs[key]

        resource_group = self.get_resource_group(self.resource_group)
        if not self.location:
            # Set default location
            self.location = resource_group.location

        changed = False
        results = dict()

        old_response = self.get_by_name()

        if old_response is not None:
            if self.state == 'present':
                update_tags, self.body['tags'] = self.update_tags(old_response['tags'])
                if update_tags or self.body['public_key'] is not None and self.body['public_key'] != old_response['public_key']:
                    changed = True
                    if not self.check_mode:
                        results = self.update_ssh_public_key(self.body)
                else:
                    results = old_response
            else:
                changed = True
                if not self.check_mode:
                    results = self.delete_ssh_public_key()
        else:
            if self.state == 'present':
                changed = True
                if not self.check_mode:
                    self.body['location'] = self.location
                    results = self.create_ssh_public_key(self.body)
            else:
                changed = False
                self.log("The SSH Public Key is not exists")

        self.results['changed'] = changed
        self.results['state'] = results

        return self.results

    def get_by_name(self):
        response = None
        try:
            response = self.compute_client.ssh_public_keys.get(self.resource_group, self.name)

        except ResourceNotFoundError as exec:
            self.log("Failed to get ssh public keys, Exception as {0}".format(exec))

        return self.to_dict(response)

    def create_ssh_public_key(self, body):
        response = None
        try:
            if body.get('public_key') is None:
                response = self.to_dict(self.compute_client.ssh_public_keys.create(self.resource_group, self.name, body))
                response.update(self.to_dict(self.compute_client.ssh_public_keys.generate_key_pair(self.resource_group, self.name)))
            else:
                response = self.to_dict(self.compute_client.ssh_public_keys.create(self.resource_group, self.name, body))
        except Exception as exc:
            self.fail("Error creating SSH Public Key {0} - {1}".format(self.name, str(exc)))

        return response

    def update_ssh_public_key(self, body):
        response = None
        try:
            response = self.compute_client.ssh_public_keys.update(self.resource_group, self.name, body)
        except Exception as exc:
            self.fail("Error updating SSH Public Key {0} - {1}".format(self.name, str(exc)))
        return self.to_dict(response)

    def delete_ssh_public_key(self):
        try:
            self.compute_client.ssh_public_keys.delete(self.resource_group, self.name)
        except Exception as exc:
            self.fail("Error deleting SSH Public Key {0} - {1}".format(self.name, str(exc)))

    def to_dict(self, body):
        results = None
        if body is not None:
            if hasattr(body, 'private_key'):
                results = dict(
                    private_key=body.private_key,
                    id=body.id,
                    public_key=body.public_key
                )
            else:
                results = dict(
                    id=body.id,
                    name=body.name,
                    location=body.location,
                    tags=body.tags,
                    public_key=body.public_key
                )
        return results


def main():
    AzureRMSshPublicKey()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_licenses
short_description: Resource module for organizations _licenses
description:
  - Manage operation update of the resource organizations _licenses.
  - Update a license.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  deviceSerial:
    description: The serial number of the device to assign this license to. Set this
      to null to unassign the license. If a different license is already active on
      the device, this parameter will control queueing/dequeuing this license.
    type: str
  licenseId:
    description: LicenseId path parameter. License ID.
    type: str
  organizationId:
    description: OrganizationId path parameter. Organization ID.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations updateOrganizationLicense
    description: Complete reference of the updateOrganizationLicense API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-organization-license
notes:
  - SDK Method used are
    organizations.Organizations.update_organization_license,
  - Paths used are
    put /organizations/{organizationId}/licenses/{licenseId},
"""

EXAMPLES = r"""
- name: Update by id
  cisco.meraki.organizations_licenses:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    deviceSerial: Q234-ABCD-5678
    licenseId: string
    organizationId: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "licenseType": "string",
      "licenseKey": "string",
      "orderNumber": "string",
      "deviceSerial": "string",
      "networkId": "string",
      "state": "string",
      "seatCount": 0,
      "totalDurationInDays": 0,
      "durationInDays": 0,
      "permanentlyQueuedLicenses": [
        {
          "id": "string",
          "licenseType": "string",
          "licenseKey": "string",
          "orderNumber": "string",
          "durationInDays": 0
        }
      ],
      "claimDate": "string",
      "activationDate": "string",
      "expirationDate": "string",
      "headLicenseId": "string"
    }
"""



#!/usr/bin/python
#
# Copyright (c) 2016 Matt Davis, <mdavis@ansible.com>
#                    Chris Houseknecht, <house@redhat.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_virtualnetwork
version_added: 0.1.0
short_description: Manage Azure virtual networks
description:
    - Create, update or delete a virtual networks. Allows setting and updating the available IPv4 address ranges
      and setting custom DNS servers. Use the M(azure.azcollection.azure_rm_subnet) module to associate subnets with a virtual network.
options:
    resource_group:
        description:
            - Name of resource group.
        required: true
        type: str
    address_prefixes_cidr:
        aliases:
            - address_prefixes
        description:
            - List of IPv4 address ranges where each is formatted using CIDR notation.
            - Required when creating a new virtual network or using I(purge_address_prefixes).
        elements: str
        type: list
    dns_servers:
        description:
            - Custom list of DNS servers.
            - The first server in the list will be treated as the Primary server. This is an explicit list.
            - Existing DNS servers will be replaced with the specified list.
            - Use the I(purge_dns_servers) option to remove all custom DNS servers and revert to default Azure servers.
        elements: str
        type: list
    location:
        description:
            - Valid Azure location. Defaults to location of the resource group.
        type: str
    name:
        description:
            - Name of the virtual network.
        required: true
        type: str
    purge_address_prefixes:
        description:
            - Use with I(state=present) to remove any existing I(address_prefixes).
        type: bool
        default: false
        aliases:
          - purge
    purge_dns_servers:
        description:
            - Use with I(state=present) to remove existing DNS servers, reverting to default Azure servers. Mutually exclusive with DNS servers.
        type: bool
        default: false
    flow_timeout_in_minutes:
        description:
            - The FlowTimeout value (in minutes) for the Virtual Network.
        type: int
    state:
        description:
            - State of the virtual network. Use C(present) to create or update and C(absent) to delete.
        default: present
        type: str
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Chris Houseknecht (@chouseknecht)
    - Matt Davis (@nitzmahone)

'''

EXAMPLES = '''
- name: Create a virtual network
  azure_rm_virtualnetwork:
    resource_group: myResourceGroup
    name: myVirtualNetwork
    address_prefixes_cidr:
      - 10.1.0.0/16
      - 172.100.0.0/16
    dns_servers:
      - 127.0.0.1
      - 127.0.0.2
    tags:
      testing: testing
      delete: on-exit

- name: Delete a virtual network
  azure_rm_virtualnetwork:
    resource_group: myResourceGroup
    name: myVirtualNetwork
    state: absent
'''
RETURN = '''
state:
    description:
        - Current state of the virtual network.
    returned: always
    type: complex
    contains:
        address_prefixes:
            description:
                - The virtual network IPv4 address ranges.
            returned: always
            type: list
            sample: [
                     "10.1.0.0/16",
                     "172.100.0.0/16"
                    ]
        dns_servers:
            description:
                - DNS servers.
            returned: always
            type: list
            sample: [
                    "127.0.0.1",
                    "127.0.0.3"
                    ]
        etag:
            description:
                - A unique read-only string that changes whenever the resource is update.
            returned: always
            type: str
            sample: 'W/"0712e87c-f02f-4bb3-8b9e-2da0390a3886"'
        id:
            description:
                - Resource ID.
            returned: always
            type: str
            sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroup/myResourceGroup/providers/
                    Microsoft.Network/virtualNetworks/myVirtualNetwork"
        location:
            description:
                - The Geo-location where the resource lives.
            returned: always
            type: str
            sample: eastus
        name:
            description:
                - Resource name.
            returned: always
            type: str
            sample: my_test_network
        provisioning_state:
            description:
                - Provisioning state of the virtual network.
            returned: always
            type: str
            sample: Succeeded
        tags:
            description:
                - Resource tags, such as { 'tags1':'value1' }.
            returned: always
            type: dict
            sample: { 'key1':'value1' }
        type:
            description:
                - Resource type.
            returned: always
            type: str
            sample: Microsoft.Network/virtualNetworks
        flow_timeout_in_minutes:
            description:
                - The FlowTimeout value (in minutes) for the Virtual Network.
            type: int
            returned: always
            sample: 8
'''

try:
    from azure.core.exceptions import ResourceNotFoundError
    import ipaddress
except ImportError:
    # This is handled in azure_rm_common
    pass

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase, CIDR_PATTERN


def virtual_network_to_dict(vnet):
    '''
    Convert a virtual network object to a dict.
    :param vnet: VirtualNet object
    :return: dict
    '''
    results = dict(
        id=vnet.id,
        name=vnet.name,
        location=vnet.location,
        type=vnet.type,
        tags=vnet.tags,
        provisioning_state=vnet.provisioning_state,
        flow_timeout_in_minutes=vnet.flow_timeout_in_minutes,
        etag=vnet.etag
    )
    if vnet.dhcp_options and len(vnet.dhcp_options.dns_servers) > 0:
        results['dns_servers'] = []
        for server in vnet.dhcp_options.dns_servers:
            results['dns_servers'].append(server)
    if vnet.address_space and len(vnet.address_space.address_prefixes) > 0:
        results['address_prefixes'] = []
        for space in vnet.address_space.address_prefixes:
            results['address_prefixes'].append(space)
    return results


class AzureRMVirtualNetwork(AzureRMModuleBase):

    def __init__(self):

        self.module_arg_spec = dict(
            resource_group=dict(type='str', required=True),
            name=dict(type='str', required=True),
            state=dict(type='str', default='present', choices=['present', 'absent']),
            location=dict(type='str'),
            address_prefixes_cidr=dict(type='list', elements='str', aliases=['address_prefixes']),
            dns_servers=dict(type='list', elements='str'),
            purge_address_prefixes=dict(type='bool', default=False, aliases=['purge']),
            purge_dns_servers=dict(type='bool', default=False),
            flow_timeout_in_minutes=dict(type='int'),
        )

        mutually_exclusive = [
            ('dns_servers', 'purge_dns_servers')
        ]

        required_if = [
            ('purge_address_prefixes', True, ['address_prefixes_cidr'])
        ]

        self.resource_group = None
        self.name = None
        self.state = None
        self.location = None
        self.address_prefixes_cidr = None
        self.purge_address_prefixes = None
        self.dns_servers = None
        self.purge_dns_servers = None
        self.flow_timeout_in_minutes = None

        self.results = dict(
            changed=False,
            state=dict()
        )

        super(AzureRMVirtualNetwork, self).__init__(self.module_arg_spec,
                                                    mutually_exclusive=mutually_exclusive,
                                                    required_if=required_if,
                                                    supports_check_mode=True)

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            setattr(self, key, kwargs[key])

        self.results['check_mode'] = self.check_mode

        resource_group = self.get_resource_group(self.resource_group)
        if not self.location:
            # Set default location
            self.location = resource_group.location

        if self.state == 'present' and self.purge_address_prefixes:
            for prefix in self.address_prefixes_cidr:
                if ':' in prefix:
                    new_prefix = prefix.split('/')
                    if int(new_prefix[1]) not in range(0, 128) or ipaddress.ip_address(new_prefix[0]).version != 6:
                        self.fail("Parameter error: invalid address prefix value {0}".format(prefix))
                elif not CIDR_PATTERN.match(prefix):
                    self.fail("Parameter error: invalid address prefix value {0}".format(prefix))
                else:
                    pass

        changed = False
        results = dict()

        try:
            self.log('Fetching vnet {0}'.format(self.name))
            vnet = self.network_client.virtual_networks.get(self.resource_group, self.name)

            results = virtual_network_to_dict(vnet)
            self.log('Vnet exists {0}'.format(self.name))
            self.log(results, pretty_print=True)
            self.check_provisioning_state(vnet, self.state)

            if self.state == 'present':
                if self.address_prefixes_cidr:
                    existing_address_prefix_set = set(vnet.address_space.address_prefixes)
                    requested_address_prefix_set = set(self.address_prefixes_cidr)
                    missing_prefixes = requested_address_prefix_set - existing_address_prefix_set
                    extra_prefixes = existing_address_prefix_set - requested_address_prefix_set
                    if len(missing_prefixes) > 0:
                        self.log('CHANGED: there are missing address_prefixes')
                        changed = True
                        if not self.purge_address_prefixes:
                            # add the missing prefixes
                            for prefix in missing_prefixes:
                                results['address_prefixes'].append(prefix)

                    if len(extra_prefixes) > 0 and self.purge_address_prefixes:
                        self.log('CHANGED: there are address_prefixes to purge')
                        changed = True
                        # replace existing address prefixes with requested set
                        results['address_prefixes'] = self.address_prefixes_cidr

                update_tags, results['tags'] = self.update_tags(results['tags'])
                if update_tags:
                    changed = True

                if self.dns_servers:
                    existing_dns_set = set(vnet.dhcp_options.dns_servers) if vnet.dhcp_options else set([])
                    requested_dns_set = set(self.dns_servers)
                    if existing_dns_set != requested_dns_set:
                        self.log('CHANGED: replacing DNS servers')
                        changed = True
                        results['dns_servers'] = self.dns_servers

                if self.purge_dns_servers and vnet.dhcp_options and len(vnet.dhcp_options.dns_servers) > 0:
                    self.log('CHANGED: purging existing DNS servers')
                    changed = True
                    results['dns_servers'] = []

                if self.flow_timeout_in_minutes and self.flow_timeout_in_minutes != vnet.flow_timeout_in_minutes:
                    self.log('CHANGED: Update flow_timeout_in_minutes')
                    changed = True
                    results['flow_timeout_in_minutes'] = self.flow_timeout_in_minutes
                else:
                    self.flow_timeout_in_minutes = vnet.flow_timeout_in_minutes
            elif self.state == 'absent':
                self.log("CHANGED: vnet exists but requested state is 'absent'")
                changed = True
        except ResourceNotFoundError:
            self.log('Vnet {0} does not exist'.format(self.name))
            if self.state == 'present':
                self.log("CHANGED: vnet {0} does not exist but requested state is 'present'".format(self.name))
                changed = True

        self.results['changed'] = changed
        self.results['state'] = results

        if self.check_mode:
            return self.results

        if changed:
            if self.state == 'present':
                if not results:
                    # create a new virtual network
                    self.log("Create virtual network {0}".format(self.name))
                    if not self.address_prefixes_cidr:
                        self.fail('Parameter error: address_prefixes_cidr required when creating a virtual network')
                    vnet_param = self.network_models.VirtualNetwork(
                        location=self.location,
                        flow_timeout_in_minutes=self.flow_timeout_in_minutes,
                        address_space=self.network_models.AddressSpace(
                            address_prefixes=self.address_prefixes_cidr
                        )
                    )
                    if self.dns_servers:
                        vnet_param.dhcp_options = self.network_models.DhcpOptions(
                            dns_servers=self.dns_servers
                        )
                    if self.tags:
                        vnet_param.tags = self.tags
                    self.results['state'] = self.create_or_update_vnet(vnet_param)
                else:
                    # update existing virtual network
                    self.log("Update virtual network {0}".format(self.name))
                    vnet_param = self.network_models.VirtualNetwork(
                        location=results['location'],
                        address_space=self.network_models.AddressSpace(
                            address_prefixes=results['address_prefixes']
                        ),
                        tags=results['tags'],
                        subnets=vnet.subnets
                    )
                    if results.get('dns_servers'):
                        vnet_param.dhcp_options = self.network_models.DhcpOptions(
                            dns_servers=results['dns_servers']
                        )
                    if self.flow_timeout_in_minutes:
                        vnet_param.flow_timeout_in_minutes = self.flow_timeout_in_minutes
                    self.results['state'] = self.create_or_update_vnet(vnet_param)
            elif self.state == 'absent':
                self.delete_virtual_network()
                self.results['state']['status'] = 'Deleted'

        return self.results

    def create_or_update_vnet(self, vnet):
        try:
            poller = self.network_client.virtual_networks.begin_create_or_update(self.resource_group, self.name, vnet)
            new_vnet = self.get_poller_result(poller)
        except Exception as exc:
            self.fail("Error creating or updating virtual network {0} - {1}".format(self.name, str(exc)))
        return virtual_network_to_dict(new_vnet)

    def delete_virtual_network(self):
        try:
            poller = self.network_client.virtual_networks.begin_delete(self.resource_group, self.name)
            result = self.get_poller_result(poller)
        except Exception as exc:
            self.fail("Error deleting virtual network {0} - {1}".format(self.name, str(exc)))
        return result


def main():
    AzureRMVirtualNetwork()


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_postgresqlflexiblevirtualendpoint_info
version_added: "3.6.0"
short_description: Get or list Azure PostgreSQL Flexible Virtual Endpoints facts
description:
    - Get or list facts of PostgreSQL Flexible Virtual Endpoints.

options:
    resource_group:
        description:
            - The name of the resource group that contains the resource. You can obtain this value from the Azure Resource Manager API or the portal.
        type: str
        required: True
    server_name:
        description:
            - The name of the post gresql server.
        type: str
        required: True
    virtual_endpoint_name:
        description:
            - The name of the post gresql virtual endpoint.
        type: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - magodo (@magodo)
    - xuzhang3 (@xuzhang3)
    - Fred-sun (@Fred-sun)

'''

EXAMPLES = '''
- name: List instance of PostgreSQL Flexible Virtual Endpoints by server name
  azure_rm_postgresqlflexiblevirtualendpoint_info:
    resource_group: myResourceGroup
    server_name: server_name

- name: Get instances of PostgreSQL Flexible Virtual Endpoints
  azure_rm_postgresqlflexiblevirtualendpoint_info:
    resource_group: myResourceGroup
    server_name: server_name
    virutal_endpoint_name: vendpoint
'''

RETURN = '''
virtual_endpoints:
    description:
        - A list of dictionaries containing facts for PostgreSQL Flexible Virtual Endpoints.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Resource ID of the postgresql flexible virtual endpoints.
            returned: always
            type: str
            sample: "/subscriptions/xxx-xxx/resourceGroups/testRG/providers/Microsoft.DBforPostgreSQL/flexibleServers/postsql01/virtualendpoints/vendpoint"
        virtual_endpoint_name:
            description:
                - Resource name.
            returned: always
            type: str
            sample: vendpoint
        endpoint_type:
            description:
                - The endpoint type for the virtual endpoint.
            returned: always
            type: str
            sample: ReadWrite
        resource_group:
            description:
                - The resoure group name.
            returned: always
            type: str
            sample: myResourceGroup
        members:
            description:
                - List of members for a virtual endpoint.
            returned: always
            type: list
            sample: ['postsqlrpfx01']
        server_name:
            description:
                - The Post gresql flexibeserver name.
            returned: always
            type: str
            sample: postsql01
        virtual_endpoints:
            description:
                - List of virtual endpoints for a server.
            returned: always
            type: list
            sample: ["vendpoint.writer.postgres.database.azure.com", "vendpoint.reader.postgres.database.azure.com"]
        type:
            description:
                - The type of the resource.
            returned: always
            type: str
            sample: Microsoft.DBforPostgreSQL/flexibleServers/virtualendpoints
'''


try:
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMPostgreSqlFlexibleVirtualEndpointInfo(AzureRMModuleBase):
    def __init__(self):
        # define user inputs into argument
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            server_name=dict(
                type='str',
                required=True
            ),
            virtual_endpoint_name=dict(
                type='str'
            ),
        )
        # store the results of the module operation
        self.results = dict(
            changed=False
        )
        self.resource_group = None
        self.virtual_endpoint_name = None
        self.server_name = None
        super(AzureRMPostgreSqlFlexibleVirtualEndpointInfo, self).__init__(self.module_arg_spec,
                                                                           supports_check_mode=True,
                                                                           supports_tags=False,
                                                                           facts_module=True)

    def exec_module(self, **kwargs):
        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if self.virtual_endpoint_name:
            self.results['virtual_endpoints'] = self.get()
        else:
            self.results['virtual_endpoints'] = self.list_all()
        return self.results

    def get(self):
        response = None
        results = []
        try:
            response = self.postgresql_flexible_client.virtual_endpoints.get(resource_group_name=self.resource_group,
                                                                             server_name=self.server_name,
                                                                             virtual_endpoint_name=self.virtual_endpoint_name)
            self.log("Response : {0}".format(response))
        except ResourceNotFoundError:
            self.log('Could not get virtual endpoint facts for PostgreSQL Flexible Server.')

        if response:
            results.append(self.format_item(response))

        return results

    def list_all(self):
        response = None
        results = []
        try:
            response = self.postgresql_flexible_client.virtual_endpoints.list_by_server(resource_group_name=self.resource_group,
                                                                                        server_name=self.server_name)
            self.log("Response : {0}".format(response))
        except Exception as ec:
            self.log('Could not list virtual endpoints facts for PostgreSQL Flexible Servers.')

        if response:
            for item in response:
                results.append(self.format_item(item))

        return results

    def format_item(self, item):
        return dict(
            resource_group=self.resource_group,
            server_name=self.server_name,
            id=item.id,
            endpoint_type=item.endpoint_type,
            members=item.members,
            virtual_endpoint_name=item.name,
            type=item.type,
            virtual_endpoints=item.virtual_endpoints
        )


def main():
    AzureRMPostgreSqlFlexibleVirtualEndpointInfo()


if __name__ == '__main__':
    main()
